//=========================================================================
//
//	FILE:				Undo_ContentView.cpp
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			CP_View class that draws rectangles when the user
//						clicks on the view to demonstrate the ability to undo/redo
//						these actions.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 6 $
//						Latest Checkout:  $Date: 2005-09-05 19:26:28 -0700 (Mon, 05 Sep 2005) $
//
//=========================================================================

	// UndoActions
#include "Undo_ContentView.h"
#include "Undo_TestView.h"

	// CPLAT_Framework
#include "CP_CarbonEvents.h"
#include "CP_Color.h"
#include "CP_Enums.h"
#include "CP_Graphics.h"
#include "CP_Numeric.h"
#include "CP_OSStatus.h"
#include "CP_Undo.h"
#include "CP_Window.h"
#include "CP_WindowEvents.h"

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Construction / Destruction ==
#pragma mark -
#endif

//=========================================================================
//	FUNCTION:	Undo_ContentView
//	
//	VISIBLITY:	Public
//=========================================================================
/*!
*	Constructor.
*
*	@param		inSuperview			The subview handler that contains this pane.
*	@param		inUpperLeft			Upper left corner of pane in releation to its superview.
*	@param		inContentSize		Content size of the pane.
*	
*	@exception	none
*/
Undo_ContentView::Undo_ContentView( CPLAT::CP_View *inSuperview,
	const CPLAT::CP_Point& inUpperLeft, const CPLAT::CP_Size& inContentSize)
		: CP_View( inSuperview, inUpperLeft, inContentSize, true )
{
	InitEventHandler( GetEventHandler() );
}

//=========================================================================
//	FUNCTION:	~Undo_ContentView
//	
//	VISIBLITY:	Public
//=========================================================================
/*!
*	Destructor.
*	
*	@exception	none
*/
Undo_ContentView::~Undo_ContentView() throw()
{
}

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Drawing ===
#pragma mark -
#endif

//=========================================================================
//	FUNCTION:	OnDraw
//	
//	VISIBLITY:	Protected
//=========================================================================
/*!
*	Called when the view needs to update the visual appearance of itself. In
*	this demo we use the CP_Graphics class to display the rectangles.
*
*	@param		inCanvas			The CP_Canvas to draw into.
*	@param		inUpdateArea		The area that needs to be drawn.
*	
*	@exception	none
*
*	@return		void
*/
void Undo_ContentView::OnDraw( CPLAT::CP_Canvas& inCanvas, CPLAT::CP_Region& /*inUpdateArea*/ )
{
	CPLAT::CP_Rect			localBounds = GetLocalBounds();
	CPLAT::CP_Graphics		graphics( inCanvas );
	CPLAT::CP_Color			aliceBlueColor = CPLAT::CP_Color::GetColor( CPLAT::CP_Color::INDEX_aliceblue );
	CPLAT::CP_ARGBColor		graphicsAliceBlueColor( aliceBlueColor.GetRGBColor() );
	
	graphics.SetStrokeColor( graphicsAliceBlueColor);
	graphics.SetFillColor( graphicsAliceBlueColor);

	graphics.FillRect( localBounds );
}

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Events ==
#pragma mark -
#endif

//=========================================================================
//	FUNCTION:	InitEventHandler
//	
//	VISIBLITY:	Protected
//=========================================================================
/*!
*	Initializes the event handler by installing a mouse down or control clicked
*	handler to add a new rect for each mouse click.
*
*	@param		inEventHandler			The event handler to add the events to.
*	
*	@exception	none
*
*	@return		void
*/
void Undo_ContentView::InitEventHandler( CPLAT::CP_EventHandler& inEventHandler )
{
	CPLAT::CP_EventSignal			*controlHitSignal = CP_new CPLAT::CP_EventSignal();
	
	#if TARGET_OS_MAC
		controlHitSignal->connect( slot(this, &Undo_ContentView::HandleControlHitEvent) );
		inEventHandler.Add( kEventClassControl, kEventControlClick, controlHitSignal );
	#endif

	#if TARGET_OS_WIN32		
		controlHitSignal->connect( CPLAT::CP_SigSlot::slot(this, &Undo_ContentView::HandleControlHitEvent) );
		inEventHandler.Add( CPLAT::kCP_EventClassMouse, CPLAT::kCP_Event_MouseDown, controlHitSignal, GetOSControl() );
	#endif
}

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Slots ==
#pragma mark -
#endif

//=========================================================================
//	FUNCTION:	HandleControlHitEvent
//	
//	VISIBLITY:	Private
//=========================================================================
/*!
*	Slot called when the user clicks on the view. We use this opportunity to
*	create a new undo / redo action.
*
*	@param		inEvent					The control clicked/mouse down event.
*	
*	@exception	none
*
*	@return		void
*/
void Undo_ContentView::HandleControlHitEvent( CPLAT::CP_Event_Base& inEvent )
{
	CPLAT::OSStatus				status = CP_eventNotHandledErr;	// so the click will get to our content handler controls

	#if TARGET_OS_MAC
		CPLAT::CP_Event<kEventClassControl, kEventControlClick>	controlClickEvent( inEvent );
		Point						qdPoint;
		ControlPartCode 			partCode = kControlNoPart;
		
		qdPoint.h = CPLAT::numeric_cast<SInt16>(controlClickEvent.fMouseLocation.h());
		qdPoint.v = CPLAT::numeric_cast<SInt16>(controlClickEvent.fMouseLocation.v());

		CPLAT::CP_Point				mouseLocation = controlClickEvent.fMouseLocation;
		GlobalToPort( mouseLocation );
		PortToLocal( mouseLocation );

		CPLAT::CP_String			undoText( CP_T("Add View") );
		CreateViewAction			*createViewAction = CP_new CreateViewAction( undoText, mouseLocation, this );
		CPLAT::CP_Undo				undo( GetEventHandler() );
		undo.Add( createViewAction );
		undo.Commit( GetContainingWindow()->GetOSWindow() );
	#endif
	
	#if TARGET_OS_WIN32
		CPLAT::CP_Event<CPLAT::kCP_EventClassMouse, CPLAT::kCP_Event_MouseDown>	controlClickEvent( inEvent );
		CPLAT::CP_Point				mouseLocation = controlClickEvent.fMouseLocation;
		GlobalToPort( mouseLocation );
		PortToLocal( mouseLocation );

		CPLAT::CP_String			undoText( CP_T("Add View") );
		CreateViewAction			*createViewAction = CP_new CreateViewAction( undoText, mouseLocation, this );
		CPLAT::CP_Undo				undo( GetEventHandler() );
		undo.Add( createViewAction );
		undo.Commit( GetContainingWindow()->GetOSWindow() );
	#endif

	inEvent.SetStatus( status );
}

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Mac Specific ===
#pragma mark -
#endif

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Construction / Destruction ==
#pragma mark -
#endif

//=========================================================================
//	FUNCTION:	CreateViewAction
//	
//	VISIBLITY:	Public
//=========================================================================
/*!
*	Constructor.
*	
*	@exception	none
*/
CreateViewAction::CreateViewAction( CPLAT::CP_String& inUndoActionText, CPLAT::CP_Point& inMouseLocation,
	CPLAT::CP_View *inSuperview )
		: CPLAT::CP_UndoAction( inUndoActionText ),
			fMouseLocation(inMouseLocation), fSuperview(inSuperview), fCreatedView(NULL)
{
}

//=========================================================================
//	FUNCTION:	~CreateViewAction
//	
//	VISIBLITY:	Public
//=========================================================================
/*!
*	Destructor.
*	
*	@exception	none
*/
CreateViewAction::~CreateViewAction() throw()
{
}

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Undo / Redo ==
#pragma mark -
#endif

//=========================================================================
//	FUNCTION:	UndoSelf
//	
//	VISIBLITY:	Protected
//=========================================================================
/*!
*	Undo the action.
*	
*	@exception	none
*
*	@return		bool					Returns true if we were able to undo the action.
*/
bool CreateViewAction::UndoSelf()
{
	bool						result = true;
	
	if( fSuperview != NULL )
	{
		fCreatedView->Hide();
		fSuperview->DeleteSubView( fCreatedView, true, true );
		
		fCreatedView = NULL;
	}

	return( result );
}

//=========================================================================
//	FUNCTION:	RedoSelf
//	
//	VISIBLITY:	Protected
//=========================================================================
/*!
*	Redo the action.
*	
*	@exception	none
*
*	@return		bool					Returns true if we were able to redo the action.
*/
bool CreateViewAction::RedoSelf()
{
	bool						result = true;
	CPLAT::CP_Point				viewLocation( fMouseLocation.h() - 20, fMouseLocation.v() - 20 );
	CPLAT::CP_Size				viewSize( 40, 40 );
	
	fCreatedView = CP_new Undo_TestView( fSuperview, viewLocation, viewSize );
	fCreatedView->Show();
	fCreatedView->Refresh();
	
	return( result );
}

#if TARGET_OS_MAC

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Mac Specific ===
#pragma mark -
#endif

#endif	// TARGET_OS_MAC

#if TARGET_OS_WIN32

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Win Specific ===
#pragma mark -
#endif

#endif	// TARGET_OS_WIN32

#if TARGET_OS_LINUX

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Linux Specific ===
#pragma mark -
#endif

#endif	// TARGET_OS_LINUX
