//=========================================================================
//
//	FILE:				CP_DateTime.h
//	
//						Created by Jens Miltner on 09.12.04.
//	
//	PURPOSE:			An object representing a specific point in time (date + time).
//	
//	HISTORY:			1st Version - January 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 266 $
//						Latest Checkout:  $Date: 2005-08-29 21:53:00 -0700 (Mon, 29 Aug 2005) $
//
//=========================================================================
#ifndef _H_CP_DateTime
#define _H_CP_DateTime

// CPLAT_Framework
#include "CP_String.h"
#include "CP_Types.h"
#include "CP_Date.h"

// Standard Library
#include <ctime>

#if TARGET_OS_MAC
	#ifdef __CF_USE_FRAMEWORK_INCLUDES__
		#include <CoreFoundation/CFDate.h>
	#endif
#endif

CPLAT_Begin_Namespace_CPLAT

class CP_Date;
class CP_Time;
class CP_TimeZone;

/*! @brief Time interval in seconds;
 *
 *  Represents the time interval between two dates in seconds. May include fractional part,
 *  in case the system supports fractional seconds in a date/time.\n
 *	You can use standard numerical methods to compare the time intervals and perform operations
 *  on them.
 */
typedef double CP_TimeInterval;

const CP_TimeInterval CP_TimeIntervalMillisecond = 0.001;
const CP_TimeInterval CP_TimeIntervalSecond = 1.0;
const CP_TimeInterval CP_TimeIntervalMinute = CP_TimeIntervalSecond * 60.0;
const CP_TimeInterval CP_TimeIntervalHour = CP_TimeIntervalMinute * 60.0;
const CP_TimeInterval CP_TimeIntervalDay = CP_TimeIntervalHour * 24.0;


/*! @brief Structure used to represent a point in time using the Gregorian calendar.
 *
 */
struct CP_GregorianDate
{
	UInt32	year;
	UInt8	month;
	UInt8	day;
	UInt8	hour;
	UInt8	minute;
	double	second;		// this member keeps the seconds and miliseconds (if any)
	
	CP_GregorianDate() {}
	CP_GregorianDate(const CP_GregorianDate& src) : year(src.year), month(src.month), day(src.day), hour(src.hour), minute(src.minute), second(src.second) {}
	CP_GregorianDate(UInt32	inYear,
					 UInt8	inMonth,
					 UInt8	inDay,
					 UInt8	inHour=0,
					 UInt8	inMinute=0,
					 double	inSecond=0.) : year(inYear), month(inMonth), day(inDay), hour(inHour), minute(inMinute), second(inSecond) {}
#if TARGET_OS_MAC
	inline CP_GregorianDate(const CFGregorianDate& inDate);
	inline CFGregorianDate GetCFGregorianDate() const;
#endif
};

/*! @brief Structure used to represent a time interval in Gregorian units.
*
*  Structure used to represent a time interval in Gregorian units. Negative values are o.k. and
*  represent negative time intervals (i.e. going backward in time).
*
*/
struct CP_GregorianUnits
{
	SInt32	years;
	SInt32	months;
	SInt32	days;
	SInt32	hours;
	SInt32	minutes;
	double	seconds;
	
	CP_GregorianUnits() {}
	CP_GregorianUnits(const CP_GregorianUnits& src) : years(src.years), months(src.months), days(src.days), hours(src.hours), minutes(src.minutes), seconds(src.seconds) {}
	CP_GregorianUnits(SInt32	inYears,
					 SInt32	inMonths,
					 SInt32	inDays,
					 SInt32	inHours=0,
					 SInt32	inMinutes=0,
					 double	inSeconds=0.) : years(inYears), months(inMonths), days(inDays), hours(inHours), minutes(inMinutes), seconds(inSeconds) {}
	static inline CP_GregorianUnits Years(SInt32 inYears);
	static inline CP_GregorianUnits Months(SInt32 inMonths);
	static inline CP_GregorianUnits Days(SInt32 inDays);
	static inline CP_GregorianUnits Hours(SInt32 inHours);
	static inline CP_GregorianUnits Minutes(SInt32 inMinutes);
	static inline CP_GregorianUnits Seconds(double inSeconds);
#if TARGET_OS_MAC
	inline CP_GregorianUnits(const CFGregorianUnits& inUnits);
	inline CFGregorianUnits GetCFGregorianUnits() const;
#endif
};


/*!
 *	@class		CP_DateTime
 *	@brief		A class for dealing with full dates (i.e. a date + time).
 *	
 *				The CP_DateTime class provides methods for performing math on full dates.
 *				Dates stored in CP_DateTime are always valid. Exceptions will be raised
 *				when trying to assign an invalid date.
 *
 *				There are also methods to return the time in a specific formatted way.
 */
class CP_EXPORT CP_DateTime
{
public:
	/*! @brief Formatting styles for date & time.
	 *
	 *  "Fuzzy" formatting styles, which depend largely on user and locale settings.\n
	 *  \li Short style is usually plain numeric, e.g. "12/13/52" or "3:30pm"\n
	 *	\li Medium is longer, such as "Jan. 12, 1952"
	 *	\li Long is longer, such as e.g. "January 12, 1952" or "3:30:32pm"
	 *	\li Full is completely specified, such as "Tuesday, April 12, 1952 AD" or "3:30:42PM PST"
	 *
	 *  \sa \ref http://oss.software.ibm.com/icu/userguide/formatDateTime.html "ICU Formatting Options"
	 */
	enum FormatStyle {
		Format_none = 0,	//!< specified part is not included in formatted string
		Format_short,		//!< use short format, usually numeric only
		Format_medium,		//!< use medium format, usually abbreviated
		Format_long,		//!< use long format
		Format_full,		//!< use full format, including every detail
		Format_ISO,			//!< use format defined by ISO 8601 (e.g. "2004-01-02Z18:34:59")
		
		DefaultDateFormat = Format_medium,
		DefaultTimeFormat = Format_medium
	};
	
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
	CP_DateTime();
	CP_DateTime(const CP_DateTime& inSrc);
	//! Construct date/time with date and time.
	CP_DateTime(const CP_Date& inDate, const CP_Time& inTime, const CP_TimeZone* inTimeZone=NULL);
	//! Construct date/time with time at midnight of the specified date.
	explicit CP_DateTime(const CP_Date& inDate, const CP_TimeZone* inTimeZone=NULL);
	//! Construct date/time with date at date origin and time as specified.
	explicit CP_DateTime(const CP_Time& inTime, const CP_TimeZone* inTimeZone=NULL);
	//! Construct date/time with the specified Gregorian date.
	CP_DateTime(const CP_GregorianDate& inGregorianDate, const CP_TimeZone* inTimeZone=NULL);
	//! Construct date/time with the specified system date
#if TARGET_CC_MSVC
	CP_DateTime( time_t inTime );
#else
	CP_DateTime( CP_STD::time_t inTime );
#endif
	//! Destructor.
	~CP_DateTime() throw();
	//@}

	//! Returns the current date & time.
	static CP_DateTime Now();
	
	//! @name Getters
	//@{
	//! Returns the date/time in Julian format in the specified time zone.
	UInt32						GetJulian(const CP_TimeZone* inTimeZone=NULL) const;
	
	//! Returns the Gregorian representation of the date/time in the specified time zone
	CP_GregorianDate			GetGregorianDate(const CP_TimeZone* inTimeZone=NULL) const;
	
	//! Returns the year of the date/time in the specified time zone.
	UInt16						GetYear(const CP_TimeZone* inTimeZone=NULL) const;
	
	//! Returns the month of the date/time in the specified time zone.
	UInt8						GetMonth(const CP_TimeZone* inTimeZone=NULL) const;
	
	//! Returns the day of the date/time in the specified time zone.
	UInt8						GetDay(const CP_TimeZone* inTimeZone=NULL) const;
	
	//! Returns the day of the week this date falls on in the specified time zone.
	UInt8						GetDayOfWeek(const CP_TimeZone* inTimeZone=NULL) const;
	
	//! Returns the day of the year this date falls on in the specified time zone.
	UInt16          GetDayOfYear(const CP_TimeZone* inTimeZone=NULL) const;
	
	//! Returns the day of week in month (e.g. 2 means 2nd Wed in July)
	UInt8           GetDayOfWeekInMonth(const CP_TimeZone* inTimeZone=NULL) const;
	
	//! Returns the week of year ( 1 to 52(53) ) this date falls in the specified time zone
	UInt8           GetWeekInYear(const CP_TimeZone* inTimeZone=NULL) const;
	
	//! Returns the week of month (1 to 5) this date falls in the specified time zone
	UInt8           GetWeekInMonth(const CP_TimeZone* inTimeZone=NULL) const;
	
	//! Returns the name of the month this date falls on in the specified time zone.
	CP_String					GetMonthString( bool inShortVersion=false, const CP_TimeZone* inTimeZone=NULL ) const;
	
	//! Returns the hour portion of the date/time in the specified time zone.
	UInt8						GetHour(const CP_TimeZone* inTimeZone=NULL) const;
	
	//! Returns the minute portion of the date/time in the specified time zone.
	SInt16						GetMinute(const CP_TimeZone* inTimeZone=NULL) const;
	
	//! Returns the seconds portion of the date/time in the specified time zone.
	UInt8						GetSecond(const CP_TimeZone* inTimeZone=NULL) const;
	
	//! Returns the milliseconds portion of the date/time in the specified time zone.
	UInt16						GetMillisecond(const CP_TimeZone* inTimeZone=NULL) const;
	
	//! Returns the date portion of the date/time in the specified time zone.
	CP_Date						GetDate(const CP_TimeZone* inTimeZone=NULL) const;
	
	//! Returns the time portion of the date/time in the specified time zone.
	CP_Time						GetTime(const CP_TimeZone* inTimeZone=NULL) const;
	//@}

	//! @name Setters
	//@{
	void						SetToNow();
	
	//! Sets the date/time to the specified Gregorian date in the defined timezone
	void						SetGregorianDate(const CP_GregorianDate& inGregorianDate, const CP_TimeZone* inTimeZone=NULL);
	
	//! Sets the year of this date/time.
	void						SetYear( UInt16 inYear, const CP_TimeZone* inTimeZone=NULL );
	
	//! Sets the month of this date/time.
	void						SetMonth( UInt8 inMonth, const CP_TimeZone* inTimeZone=NULL );
	
	//! Sets the date of this date/time.
	void						SetDay( UInt8 inDay, const CP_TimeZone* inTimeZone=NULL );
	
	//! Sets the date of the week of this date/time.
	void						SetDayOfWeek( UInt8 inDayOfWeek, const CP_TimeZone* inTimeZone=NULL );
	
	//! Sets the hour of this date/time.
	void						SetHour( UInt8 inHour, const CP_TimeZone* inTimeZone=NULL );
	
	//! Sets the minute of this date/time.
	void						SetMinute( UInt8 inMinute, const CP_TimeZone* inTimeZone=NULL );
	
	//! Sets the second of this date/time.
	void						SetSecond( UInt8 inSecond, const CP_TimeZone* inTimeZone=NULL );
	
	//! Sets the hour of this date/time.
	void						SetMillisecond( UInt16 inMillisecond, const CP_TimeZone* inTimeZone=NULL );
	
	//! Sets the date portion of the date/time.
	void						SetDate(const CP_Date& inDatePart, const CP_TimeZone* inTimeZone = NULL);
	
	//! Sets the time portion of the date/time.
	void						SetTime(const CP_Time& inTimePart, const CP_TimeZone* inTimeZone = NULL);
	//@}

	//! @name Math
	//@{
	//! Adds the specified time interval to the date/time.
	CP_DateTime operator + (CP_TimeInterval inTimeInterval) const;
	//! Adds the specified time interval to the date/time.
	CP_DateTime& operator += (CP_TimeInterval inTimeInterval);
	//! Subtracts the specified time interval from the date/time.
	CP_DateTime operator - (CP_TimeInterval inTimeInterval) const;
	//! Subtracts the specified time interval from the date/time.
	CP_DateTime& operator -= (CP_TimeInterval inTimeInterval);
	//! Calculates the time interval between two date/times.
	CP_TimeInterval operator - (const CP_DateTime& inRhs) const;
	
	//! Assign a date/time.
	CP_DateTime& operator = (const CP_DateTime& inRhs);

	//! Adds the specified number of years to the date.
	CP_DateTime&			AddGregorianUnits( const CP_GregorianUnits& inUnitsToAdd, const CP_TimeZone* inTimeZone=NULL );
	//! Adds the specified number of years to the date.
	CP_DateTime&			AddYears( SInt32 inYears, const CP_TimeZone* inTimeZone=NULL );
	//! Subtract the specified number of years from the date.
	CP_DateTime&			SubtractYears( SInt32 inYears, const CP_TimeZone* inTimeZone=NULL );
	//! Adds the specified number of months to the date.
	CP_DateTime&			AddMonths( SInt32 inMonths, const CP_TimeZone* inTimeZone=NULL );
	//! Subtract the specified number of months from the date.
	CP_DateTime&			SubtractMonths( SInt32 inMonths, const CP_TimeZone* inTimeZone=NULL );
	//! Adds the specified number of days to the date.
	CP_DateTime&			AddDays( SInt32 inDays, const CP_TimeZone* inTimeZone=NULL );
	//! Subtract the specified number of days from the date.
	CP_DateTime&			SubtractDays( SInt32 inDays, const CP_TimeZone* inTimeZone=NULL );
	//! Adds the specified number of hours to the date.
	CP_DateTime&			AddHours( SInt32 inHours, const CP_TimeZone* inTimeZone=NULL );
	//! Subtract the specified number of hours from the date.
	CP_DateTime&			SubtractHours( SInt32 inHours, const CP_TimeZone* inTimeZone=NULL );
	//! Adds the specified number of minutes to the date.
	CP_DateTime&			AddMinutes( SInt32 inMinutes, const CP_TimeZone* inTimeZone=NULL );
	//! Subtract the specified number of minutes from the date.
	CP_DateTime&			SubtractMinutes( SInt32 inMinutes, const CP_TimeZone* inTimeZone=NULL );
	//! Adds the specified number of seconds to the date.
	CP_DateTime&			AddSeconds( double inSeconds, const CP_TimeZone* inTimeZone=NULL );
	//! Subtract the specified number of seconds from the date.
	CP_DateTime&			SubtractSeconds( double inSeconds, const CP_TimeZone* inTimeZone=NULL );
	//@}

	//! @name Comparisons
	//@{
	//! compare to another date/time
	bool							operator >( const CP_DateTime& inDate ) const;
	//! compare to another date/time
	bool							operator <( const CP_DateTime& inDate ) const;
	//! compare to another date/time
	bool							operator >=( const CP_DateTime& inDate ) const;
	//! compare to another date/time
	bool							operator <=( const CP_DateTime& inDate ) const;
	//! compare to another date/time
	bool							operator ==( const CP_DateTime& inDate ) const;
	//! compare to another date/time
	bool							operator !=( const CP_DateTime& inDate ) const;
	//! compare to another date/time to be equal up to the second (but ignoring millisecond differences)
	bool IsSecondsEqual(const CP_DateTime& inRhs) const;
	//@}
	
	//! @name String conversions
	//@{
	//! Parse a date/time string.
	static CP_DateTime FromString(const CP_String& inDateString, FormatStyle inDateFormat, FormatStyle inTimeFormat, const CP_TimeZone* inTimeZone=NULL);
	//! Parse a date/time string.
	static CP_DateTime FromStringGivenFormat(const CP_String& inDateString, const CP_String& inFormatString, const CP_TimeZone* inTimeZone=NULL);

	//! Format a date.
	CP_String				Format( FormatStyle inDateFormat=DefaultDateFormat, FormatStyle inTimeFormat=DefaultTimeFormat, const CP_TimeZone* inTimeZone=NULL ) const;
	//! Format a date.
	CP_String				Format( const CP_String& inFormatString, const CP_TimeZone* inTimeZone=NULL ) const;
	//@}

	//========================================================================
	// Platform specific routines
	//========================================================================
#if TARGET_OS_MAC
public:
	enum enum_MacOSTime { MacOSTime }; //!< to distinguish Mac specific constructor
	
	//! @name Constructor / Destructor
	//@{
	//! Mac specific Constructor.
	CP_DateTime( UInt32 inMacSeconds,
				 enum_MacOSTime inMacOSTime,
				 const CP_TimeZone* inTimeZone = NULL );
	//! Mac specific Constructor.
	CP_DateTime( const LongDateRec& inDateRec, const CP_TimeZone* inTimeZone = NULL );
	//! Mac specific Constructor.
	CP_DateTime( CFDateRef inDateRef );
	//! Mac specific Constructor.
	CP_DateTime( CFAbsoluteTime inAbsoluteTime );
	//@}
	
	//! @name Getters
	//@{
	//! Get a MacOS LongDateRec of the date/time in the specified timezone.
	void						GetLongDate( LongDateRec& outDateRec, const CP_TimeZone* inTimeZone = NULL ) const;
	//! Get a MacOS seconds value of the date/time in the specified timezone.
	UInt32						GetMacSeconds(const CP_TimeZone* inTimeZone = NULL) const;
	//! Get a CoreFoundation absolute time value for the date.
	CFAbsoluteTime				GetAbsoluteTime() const		{ return fAbsoluteTime; }
	//@}
	
	//! @name Setters
	//@{
	//! Set the date/time from LongDateRec. 
	void						SetDate( const LongDateRec& inDateRec, const CP_TimeZone* inTimeZone = NULL );
	//@}
#endif
	
#if TARGET_OS_WIN32
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
	CP_DateTime( SYSTEMTIME inSystemTime, const CP_TimeZone* inTimeZone = NULL );
	//! Constructor.
	CP_DateTime( FILETIME inSystemTime ); 
	//@}
	
	//! @name Getters
	//@{
	//! Get SYSTEMTIME representation of the date/time for the given timezone.
	void						GetDate( SYSTEMTIME& inSystemTime, const CP_TimeZone* inTimeZone = NULL ) const;
	//! Get FILETIME absolute time value for the date/time.
	FILETIME					GetAbsoluteTime() const { return fAbsoluteTime; }
	//@}
	
	//! @name Setters
	//@{
	//! Set date/time from SYSTEMTIME representation for the given timezone.
	void						SetDate( const SYSTEMTIME& inSystemTime, const CP_TimeZone* inTimeZone = NULL );
	void						SetAbsoluteTime( const FILETIME& theFileTime ) { fAbsoluteTime = theFileTime; }
	
	//@}
#endif
	
#if TARGET_OS_LINUX
#endif
	
	// class data:
protected:
#if TARGET_OS_MAC
	CFAbsoluteTime		fAbsoluteTime;
#endif
#if TARGET_OS_WIN32
	FILETIME          fAbsoluteTime;
#endif	
#if TARGET_OS_LINUX
#endif
};


inline CP_GregorianUnits CP_GregorianUnits::Years(SInt32 inYears)
{
	return CP_GregorianUnits(inYears, 0, 0, 0, 0, 0.);
}

inline CP_GregorianUnits CP_GregorianUnits::Months(SInt32 inMonths)
{
	return CP_GregorianUnits(0, inMonths, 0, 0, 0, 0.);
}

inline CP_GregorianUnits CP_GregorianUnits::Days(SInt32 inDays)
{
	return CP_GregorianUnits(0, 0, inDays, 0, 0, 0.);
}

inline CP_GregorianUnits CP_GregorianUnits::Hours(SInt32 inHours)
{
	return CP_GregorianUnits(0, 0, 0, inHours, 0, 0.);
}

inline CP_GregorianUnits CP_GregorianUnits::Minutes(SInt32 inMinutes)
{
	return CP_GregorianUnits(0, 0, 0, 0, inMinutes, 0.);
}

inline CP_GregorianUnits CP_GregorianUnits::Seconds(double inSeconds)
{
	return CP_GregorianUnits(0, 0, 0, 0, 0, inSeconds);
}

#if TARGET_OS_MAC
inline CP_GregorianDate::CP_GregorianDate(const CFGregorianDate& inDate)
{
	year = (UInt32) inDate.year;
	month = (unsigned char) inDate.month;
	day = (unsigned char) inDate.day;
	hour = (unsigned char) inDate.hour;
	minute = (unsigned char) inDate.minute;
	second = inDate.second;
}
inline CFGregorianDate CP_GregorianDate::GetCFGregorianDate() const
{
	CFGregorianDate	result;
	result.year = (SInt32) year;
	result.month = (char) month;
	result.day = (char) day;
	result.hour = (char) hour;
	result.minute = (char) minute;
	result.second = (double) second;
	return result;
}
inline CP_GregorianUnits::CP_GregorianUnits(const CFGregorianUnits& inUnits)
{
	years = inUnits.years;
	months = inUnits.months;
	days = inUnits.days;
	hours = inUnits.hours;
	minutes = inUnits.minutes;
	seconds = inUnits.seconds;
}
inline CFGregorianUnits CP_GregorianUnits::GetCFGregorianUnits() const
{
	CFGregorianUnits	result;
	result.years = years;
	result.months = months;
	result.days = days;
	result.hours = hours;
	result.minutes = minutes;
	result.seconds = seconds;
	return result;
}
#endif

CPLAT_End_Namespace_CPLAT

#endif // _H_CP_DateTime

