//=========================================================================
//
//	FILE:				CP_String.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Class for handling strings.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				All strings are stored internally as Unicode data.
//
//	VERSION:			Current Revision: $Rev: 226 $
//						Latest Checkout:  $Date: 2005-07-02 15:14:01 -0700 (Sat, 02 Jul 2005) $
//
//=========================================================================

#ifndef _H_CP_String
#define _H_CP_String
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_ResourcePtr.h"
#include "CP_TextEncoder.h"
#include "CP_Types.h"

	// Standard Library
#include <iterator>
#include <stdarg.h>
#include <string>

#if !TARGET_API_MAC_CARBON
	#include "CP_M_ReferenceCounted.h"
#endif

CPLAT_Begin_Namespace_CPLAT

/*!
*	@class		CP_String
*	@brief		A class for dealing with strings.
*	
*				The CP_String class provides a std::string like interface for string data
*				used in the CPLAT framework.
*	
*				Internally all string data is stored as Unicode data, and the class provides
*				converters to convert to non-unicode as well as return the data as a Unicode
*				string.
*	
*				Under MacOS Carbon we use the CFString data type provided by the MacOS to manage
*				all data.
*	
*				Under Windows and Linux the data is stored as CPUniChar data.
*/
class CP_EXPORT CP_String
{
public:
	enum
	{
		kCP_E_String_BufferSize = 32,
		npos = MAX_UINT_32
	};

	static const CP_String		emptyString;
	
	#if TARGET_API_MAC_CARBON
		typedef CFStringRef		StringRef;
	#else
		class StringRefImp;
		typedef StringRefImp	*StringRef;
	#endif
	
	// Iterators
	#if TARGET_API_MAC_CARBON
		class __iterator_base;
		class iterator;
		class const_iterator;
	#else
		typedef CPUniChar										*iterator;
		typedef const CPUniChar									*const_iterator;
	#endif
	
	#if ! TARGET_CC_MSVC_6
		typedef std::reverse_iterator<const_iterator>			const_reverse_iterator;
		typedef std::reverse_iterator<iterator>					reverse_iterator;
	#endif

	class reference
	{
		friend class CP_String;
	#if TARGET_API_MAC_CARBON
		friend class CP_String::iterator;
	#endif
		CP_String&	fString;
		UInt32		fIndex;
	protected:
		reference(CP_String& inString, UInt32 inIndex) : fString(inString), fIndex(inIndex) {}
	public:
		reference& operator = (const reference& inRhs);
		reference& operator = (CPUniChar inRhs);
		
		operator CPUniChar () const { return static_cast<const CP_String&>(fString).at(fIndex); }
	};
	
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_String();
										CP_String(
												const char *inCString,
												const CP_TextEncoder *inTextEncoder = NULL );
										CP_String(
												const char *inCString,
												UInt32 inLength,
												const CP_TextEncoder *inTextEncoder = NULL );
										CP_String(
												UInt32 inCharCount,
												char inChar,
												const CP_TextEncoder *inTextEncoder = NULL);
										CP_String( const CPUniChar *inWString );
										CP_String(
												const CPUniChar *inWString,
												UInt32 inLength );
										CP_String(
												UInt32 inCharCount,
												CPUniChar inChar );
										CP_String(
												const unsigned char *inPString,
												const CP_TextEncoder *inTextEncoder = NULL );
										CP_String(
												const std::string& inString,
												const CP_TextEncoder *inTextEncoder = NULL );
										CP_String(
												const std::wstring& inString );
										CP_String(
												const CP_String& inString,
												UInt32 inStartIndex = 0,
												UInt32 inLength = npos );
	//! Destructor.
	virtual								~CP_String();
	//@}
	
	//! @name Creating strings using printf-style format + parameters
	//@{
	#if TARGET_CC_MSVC
		static CP_String				StringWithFormat( CP_String inFormat, ... );
	#else
		#if TARGET_CC_GNUC || __MWERKS__ >= 0x3200
			static CP_String			StringWithFormat( const CP_String& inFormat, ... );
		#else
			static CP_String			StringWithFormat( const char *inFormat, ... );
		#endif
	#endif
		static CP_String				StringWithFormatAndArguments(
												const CP_String& inFormat,
												va_list args );
	//@}
	
	//! @name Accessors
	//@{
			const char					*c_str(bool inLossy=false, const CP_TextEncoder *inTextEncoder = NULL) const;
			const CPUniChar				*w_str() const;
			const char					*utf8_str() const;

			CPUniChar 					operator[](UInt32 inIndex) const;
			reference 					operator[](UInt32 inIndex)					{ return reference(*this, inIndex); }
		
			CPUniChar 					at(UInt32 inIndex) const;
			reference 					at(UInt32 inIndex)							{ return reference(*this, inIndex); }

			CPUniChar 					front() const;
			reference 					front()										{ return reference(*this, 0); }
			CPUniChar 					back() const;
			reference 					back()										{ return reference(*this, length()-1); }
			
			StringRef					Ref() const;
	//@}

	//! @name Appending
	//@{
			CP_String&					operator += ( char inChar );
			CP_String&					operator += ( unsigned char inChar );
			CP_String&					operator += ( CPUniChar inChar );

										/*!
										*	Adds the passed in c style (NULL terminated) string to the end of our string.
										*	
										*	@param		inChar				The c style (NULL terminated) string to be added to the string.
										*	
										*	@exception	none
										*	
										*	@return		CP_String&			Returns a reference to ourselves.
										*/
			CP_String&					operator += ( const char *inString )				{ return( append( inString ) ); }

										/*!
										*	Adds the passed in Unicode (NULL terminated) string to the end of our string.
										*	
										*	@param		inChar				The Unicode (NULL terminated) string to be added to the string.
										*	
										*	@exception	none
										*	
										*	@return		CP_String&			Returns a reference to ourselves.
										*/
			CP_String&					operator += ( const CPUniChar *inString )				{ return( append( inString ) ); }

										/*!
										*	Adds the passed in CP_String string to the end of our string.
										*	
										*	@param		inChar				The CP_String string to be added to the string.
										*	
										*	@exception	none
										*	
										*	@return		CP_String&			Returns a reference to ourselves.
										*/
			CP_String&					operator += ( const CP_String& inString )			{ return( append( inString ) ); }

			CP_String&					append( char ch, 
												const CP_TextEncoder *inTextEncoder = NULL );
			CP_String&					append( unsigned char ch, 
												const CP_TextEncoder *inTextEncoder = NULL );
			CP_String&					append( CPUniChar ch );
			CP_String&					append(
												UInt32 inCount,
												CPUniChar inChar );
			CP_String&					append( const char *inString, 
												const CP_TextEncoder *inTextEncoder = NULL );
			CP_String&					append(
												const char *inString,
												UInt32 inLength, 
											    const CP_TextEncoder *inTextEncoder = NULL);
			CP_String&					append( const CPUniChar *inString );
			CP_String&					append(
												const CPUniChar *inString,
												UInt32 inLength);
			CP_String&					append(
												const CP_String& inString,
												UInt32 inStartIndex = 0,
												UInt32 inLength = npos );

										/*!
										*	Add the CPUniChar to the end of this string.
										*	
										*	@param		inChar				Unicode character being added.
										*	
										*	@exception	none
										*
										*	@return		void
										*/
			void						push_back( CPUniChar inChar )							{ resize( (UInt32)(length() + 1), inChar ); }
	//@}

	//! @name Assignment
	//@{
										/*!
										*	Resets the contents of this string to the passed in char.
										*	
										*	@param		inChar				char
										*	
										*	@exception	none
										*	
										*	@return		CP_String&			Returns a reference to ourselves.
										*/
			CP_String&					operator = ( char inChar )							{ return( assign( inChar ) ); }

										/*!
										*	Resets the contents of this string to the passed in Unicode characer.
										*	
										*	@param		inChar				Unicode character
										*	
										*	@exception	none
										*	
										*	@return		CP_String&			Returns a reference to ourselves.
										*/
			CP_String&					operator = ( CPUniChar inChar )						{ return( assign( inChar ) ); }

										/*!
										*	Resets the contents of this string to the passed in char * to a c style
										*	(NULL terminated) string.
										*	
										*	@param		inChar				c style (NULL terminated) string.
										*	
										*	@exception	none
										*	
										*	@return		CP_String&			Returns a reference to ourselves.
										*/
			CP_String&					operator = ( const char *inString )					{ return( assign( inString ) ); }

										/*!
										*	Resets the contents of this string to the passed in Unicode (NULL terminated) string.
										*	
										*	@param		inChar				Unicode (NULL terminated) string.
										*	
										*	@exception	none
										*	
										*	@return		CP_String&			Returns a reference to ourselves.
										*/
			CP_String&					operator = ( const CPUniChar *inString )				{ return( assign( inString ) ); }

										/*!
										*	Resets the contents of this string to the passed in CP_String.
										*	
										*	@param		inString			CP_String string to replace contents with.
										*	
										*	@exception	none
										*	
										*	@return		CP_String&			Returns a reference to ourselves.
										*/
			CP_String&					operator = ( const CP_String& inString )			{ return( assign( inString ) ); }

			CP_String&					assign( char inChar, 
												const CP_TextEncoder *inTextEncoder = NULL );
			CP_String&					assign( CPUniChar inChar );
			CP_String& 					assign(
												UInt32 inCount,
											    char inChar, 
											    const CP_TextEncoder *inTextEncoder = NULL );
			CP_String& 					assign(
												UInt32 inCount,
												CPUniChar inChar );
			CP_String& 					assign( const char *inString, 
												const CP_TextEncoder *inTextEncoder = NULL );
			CP_String& 					assign(
												const char *inString,
											    UInt32 inLength, 
											    const CP_TextEncoder *inTextEncoder = NULL );
			CP_String& 					assign( const CPUniChar *inString);
			CP_String& 					assign(
												const CPUniChar *inString,
												UInt32 inLength );
			CP_String& 					assign(
												const CP_String& inString, 
												UInt32 inStartIndex = 0, 
												UInt32 inLength = npos );
	//@}

	//! @name Concatenation
	//@{
	friend 	CP_String 					operator + (
												const CP_String& inLeftSide,
												const CP_String& inRightSide );
	friend 	CP_String					operator + (
												const CPUniChar *inLeftSide,
												const CP_String& inRightSide );
	friend 	CP_String 					operator + (
												CPUniChar inLeftSide,
												const CP_String& inRightSide );
	friend 	CP_String 					operator + (
												const CP_String& inLeftSide,
												const CPUniChar *inRightSide );
	friend 	CP_String 					operator + (
												const CP_String& inLeftSide,
												CPUniChar inRightSide );
	//@}

	//! @name Comparisons
	//@{
										/*!
										*	Compares the passed in inLeftSide character to the contents of the passed in CP_String.
										*	
										*	@param		inLeftSide			Character to compare against second parameter.
										*	@param		inRightSide			CP_String that is compared against.
										*	
										*	@exception	none
										*
										*	@return		bool				Returns true if inLeftSide == inRightSide, otherwise false is returned.
										*/
	friend 	bool 						operator == (
												char inLeftSide,
												const CP_String& inRightSide )				{ return( inRightSide.length() == 1 && inRightSide.operator[](0) == inLeftSide ); }

										/*!
										*	Compares the passed in inLeftSide Unicode character string to the contents of the passed in CP_String.
										*	
										*	@param		inLeftSide			Character string to compare against second parameter.
										*	@param		inRightSide			CP_String that is compared against.
										*	
										*	@exception	none
										*	
										*	@return		bool				Returns true if inLeftSide == inRightSide, otherwise false is returned.
										*/
	friend 	bool 						operator == (
												const char *inLeftSide,
												const CP_String& inRightSide )				{ return( inRightSide.compare(inLeftSide) == 0 ); }

										/*!
										*	Compares the passed in inLeftSide character to the contents of the passed in CP_String.
										*	
										*	@param		inLeftSide			Unicode character to compare against second parameter.
										*	@param		inRightSide			CP_String that is compared against.
										*	
										*	@exception	none
										*
										*	@return		bool				Returns true if inLeftSide == inRightSide, otherwise false is returned.
										*/
	friend 	bool 						operator == (
												CPUniChar inLeftSide,
												const CP_String& inRightSide )				{ return( inRightSide.length() == 1 && inRightSide.operator[](0) == inLeftSide ); }

										/*!
										*	Compares the passed in inLeftSide Unicode character string to the contents of the passed in CP_String.
										*	
										*	@param		inLeftSide			Unicode character string to compare against second parameter.
										*	@param		inRightSide			CP_String that is compared against.
										*	
										*	@exception	none
										*	
										*	@return		bool				Returns true if inLeftSide == inRightSide, otherwise false is returned.
										*/
	friend 	bool 						operator == (
												const CPUniChar *inLeftSide,
												const CP_String& inRightSide )				{ return( inRightSide.compare(inLeftSide) == 0 ); }

										/*!
										*	Compares the passed in inLeftSide CP_String to the Unicode character.
										*	
										*	@param		inLeftSide			CP_String to compare against second parameter.
										*	@param		inRightSide			Unicode character that is compared against.
										*	
										*	@exception	none
										*	
										*	@return		bool				Returns true if inLeftSide == inRightSide, otherwise false is returned.
										*/
	friend 	bool 						operator == (
												const CP_String& inLeftSide,
												CPUniChar inRightSide )						{ return( inLeftSide.length() == 1 && inLeftSide.operator[](0) == inRightSide ); }

										/*!
										*	Compares the passed in inLeftSide CP_String to the Unicode character.
										*	
										*	@param		inLeftSide			CP_String to compare against second parameter.
										*	@param		inRightSide			char * string (NULL terminated) that is compared against.
										*	
										*	@exception	none
										*	
										*	@return		bool				Returns true if inLeftSide == inRightSide, otherwise false is returned.
										*/
	friend 	bool 						operator == (
												const CP_String& inLeftSide,
												const char *inRightSide)					{ return( inLeftSide.compare(inRightSide) == 0 ); }

										/*!
										*	Compares the passed in inLeftSide CP_String to the Unicode character string.
										*	
										*	@param		inLeftSide			CP_String to compare against second parameter.
										*	@param		inRightSide			Unicode character string (NULL terminated) that is compared against.
										*	
										*	@exception	none
										*	
										*	@return		bool				Returns true if inLeftSide == inRightSide, otherwise false is returned.
										*/
	friend 	bool 						operator == (
												const CP_String& inLeftSide,
												const CPUniChar *inRightSide)					{ return( inLeftSide.compare(inRightSide) == 0 ); }

										/*!
										*	Compares the passed in inLeftSide CP_String to the passed in CP_String.
										*	
										*	@param		inLeftSide			CP_String to compare against second parameter.
										*	@param		inRightSide			CP_String that is compared against.
										*	
										*	@exception	none
										*	
										*	@return		bool				Returns true if inLeftSide == inRightSide, otherwise false is returned.
										*/
	friend 	bool 						operator == (
												const CP_String& inLeftSide,
												const CP_String& inRightSide)				{ return( inLeftSide.compare(inRightSide) == 0 ); }
	
										/*!
										*	Compares the passed in inLeftSide character to the contents of the passed in CP_String.
										*	
										*	@param		inLeftSide			Unicode character to compare against second parameter.
										*	@param		inRightSide			CP_String that is compared against.
										*	
										*	@exception	none
										*	
										*	@return		bool				Returns true if inLeftSide != inRightSide, otherwise false is returned.
										*/
	friend 	bool 						operator != (
												CPUniChar inLeftSide,
												const CP_String& inRightSide )				{ return( inRightSide.length() != 1 || inRightSide.operator[](0) != inLeftSide ); }

										/*!
										*	Compares the passed in inLeftSide Unicode character string to the contents of the passed in CP_String.
										*	
										*	@param		inLeftSide			Unicode character string to compare against second parameter.
										*	@param		inRightSide			CP_String that is compared against.
										*	
										*	@exception	none
										*	
										*	@return		bool				Returns true if inLeftSide != inRightSide, otherwise false is returned.
										*/
	friend 	bool 						operator != (
												const CPUniChar *inLeftSide,
												const CP_String& inRightSide ) 				{ return( inRightSide.compare(inLeftSide) != 0 ); }

										/*!
										*	Compares the passed in inLeftSide CP_String to the Unicode character.
										*	
										*	@param		inLeftSide			CP_String to compare against second parameter.
										*	@param		inRightSide			Unicode character that is compared against.
										*	
										*	@exception	none
										*	
										*	@return		bool				Returns true if inLeftSide != inRightSide, otherwise false is returned.
										*/
	friend 	bool 						operator != (
												const CP_String& inLeftSide,
												CPUniChar inRightSide )						{ return( inLeftSide.length() != 1 || inLeftSide.operator[](0) != inRightSide ); }

										/*!
										*	Compares the passed in inLeftSide CP_String to the Unicode character.
										*	
										*	@param		inLeftSide			CP_String to compare against second parameter.
										*	@param		inRightSide			char * string (NULL terminated) that is compared against.
										*	
										*	@exception	none
										*	
										*	@return		bool				Returns true if inLeftSide != inRightSide, otherwise false is returned.
										*/
	friend 	bool 						operator != (
												const CP_String& inLeftSide,
												const char *inRightSide )					{ return( inLeftSide.compare(inRightSide) != 0 ); }

										/*!
										*	Compares the passed in inLeftSide CP_String to the Unicode character string.
										*	
										*	@param		inLeftSide			CP_String to compare against second parameter.
										*	@param		inRightSide			Unicode character string (NULL terminated) that is compared against.
										*	
										*	@exception	none
										*	
										*	@return		bool				Returns true if inLeftSide != inRightSide, otherwise false is returned.
										*/
	friend 	bool 						operator != (
												const CP_String& inLeftSide,
												const CPUniChar *inRightSide )				{ return( inLeftSide.compare(inRightSide) != 0 ); }

										/*!
										*	Compares the passed in inLeftSide CP_String to the passed in CP_String.
										*	
										*	@param		inLeftSide			CP_String to compare against second parameter.
										*	@param		inRightSide			CP_String that is compared against.
										*	
										*	@exception	none
										*	
										*	@return		bool				Returns true if inLeftSide != inRightSide, otherwise false is returned.
										*/
	friend 	bool 						operator != (
												const CP_String& inLeftSide,
												const CP_String& inRightSide )				{ return( inLeftSide.compare(inRightSide) != 0 ); }
				
										/*!
										*	Compares the passed in inLeftSide CP_String to the passed in CP_String.
										*	
										*	@param		inLeftSide			CP_String to compare against second parameter.
										*	@param		inRightSide			CP_String that is compared against.
										*	
										*	@exception	none
										*	
										*	@return		bool				Returns true if inLeftSide < inRightSide, otherwise false is returned.
										*/
	friend 	bool 						operator < (
												const CP_String& inLeftSide,
												const CP_String& inRightSide )				{ return( inLeftSide.compare(inRightSide) < 0 ); }

										/*!
										*	Compares the passed in inLeftSide CP_String to the passed in CP_String.
										*	
										*	@param		inLeftSide			CP_String to compare against second parameter.
										*	@param		inRightSide			CP_String that is compared against.
										*	
										*	@exception	none
										*	
										*	@return		bool				Returns true if inLeftSide <= inRightSide, otherwise false is returned.
										*/
	friend 	bool 						operator <= (
												const CP_String& inLeftSide,
												const CP_String& inRightSide )				{ return( inLeftSide.compare(inRightSide) <= 0 ); }

										/*!
										*	Compares the passed in inLeftSide CP_String to the passed in CP_String.
										*	
										*	@param		inLeftSide			CP_String to compare against second parameter.
										*	@param		inRightSide			CP_String that is compared against.
										*	
										*	@exception	none
										*	
										*	@return		bool				Returns true if inLeftSide > inRightSide, otherwise false is returned.
										*/
	friend 	bool 						operator > (
												const CP_String& inLeftSide,
												const CP_String& inRightSide )				{ return( inLeftSide.compare(inRightSide) > 0 ); }

										/*!
										*	Compares the passed in inLeftSide CP_String to the passed in CP_String.
										*	
										*	@param		inLeftSide			CP_String to compare against second parameter.
										*	@param		nRightSide			CP_String that is compared against.
										*	
										*	@exception	none
										*	
										*	@return		bool				Returns true if inLeftSide >= inRightSide, otherwise false is returned.
										*/
	friend 	bool 						operator >= (
												const CP_String& inLeftSide,
												const CP_String& inRightSide )				{ return( inLeftSide.compare(inRightSide) >= 0 ); }
	//@}

	//! @name Compare
	//@{
			SInt32 						compare( const char *inChar, 
												 const CP_TextEncoder *inTextEncoder = NULL ) const;
			SInt32 						compare( const CPUniChar *inChar ) const;
			SInt32 						compare( const CP_String& inString ) const;

			SInt32 						compare(
												UInt32 inIndex,
												UInt32 inCount,
												const CPUniChar *inString,
												UInt32 inLength ) const;
			SInt32 						compare(
												UInt32 inIndex,
												UInt32 inCount,
												const CP_String& inString,
												UInt32 inStartIndex = 0,
												UInt32 inLength = npos ) const;
	//@}

	//! @name Erasing
	//@{
			CP_String&					erase(
												UInt32 inIndex = 0,
												UInt32 inCount = npos ); 

										/*!
										*	Erases the last character in the string.
										*	
										*	@exception	none
										*	
										*	@return		void
										*/
			void						pop_back()									{ erase( (UInt32) (length() - 1) ); }
	//@}

	//! @name Finding
	//@{
			UInt32 						find(
												CPUniChar inChar,
												UInt32 inIndex = 0 ) const;
			UInt32 						find(
												const char *inString,
												UInt32 inIndex = 0, 
												const CP_TextEncoder *inTextEncoder = NULL ) const;
			UInt32 						find(
												const char *inString,
												UInt32 inIndex,
												UInt32 inLength, 
												const CP_TextEncoder *inTextEncoder = NULL ) const;
			UInt32 						find(
												const CPUniChar *inString,
												UInt32 inndex = 0) const;
			UInt32 						find(
												const CPUniChar *inString,
												UInt32 inIndex,
												UInt32 inLength ) const;
			UInt32 						find(
												const CP_String& inString,
												UInt32 inndex = 0 ) const;

			UInt32 						rfind(
												CPUniChar inChar,
												UInt32 inIndex = npos ) const;
			UInt32 						rfind(
												const char *inString,
												UInt32 inIndex = npos, 
												const CP_TextEncoder *inTextEncoder = NULL ) const;
			UInt32 						rfind(
												const char *inString,
												UInt32 inIndex,
												UInt32 inLength, 
												const CP_TextEncoder *inTextEncoder = NULL ) const;
			UInt32 						rfind(
												const CPUniChar *inString,
												UInt32 inIndex = npos ) const;
			UInt32 						rfind(
												const CPUniChar *inString,
												UInt32 inIndex,
												UInt32 inLength ) const;
			UInt32 						rfind(
												const CP_String& inString,
												UInt32 inIndex = npos ) const;

										/*!
										*	Return the index of the first occurrance of inChar in this string.
										*	
										*	@param		inChar				Unicode character we are looking for.
										*	@param		inIndex				Starting index to being search at, default is the beginning.
										*	
										*	@exception	none
										*	
										*	@return		UInt32				Location of first occurrance of inChar in this string. npos if not found.
										*/
			UInt32						find_first_of(
												CPUniChar inChar,
												UInt32 inIndex = 0 ) const					{ return( find_first_of( CP_String(1, inChar), inIndex ) ); }

										/*!
										*	Return the index of the first occurrance of inSting in this string.
										*	
										*	@param		inString			c style (NULL terminated) we are looking for.
										 *	@param		inIndex				Starting index to being search at, default is the beginning.
										 *	@param		inTextEncoder		Text encoder used to convert c style string into UTF-16 representation if needed.
										*	
										*	@exception	none
										*	
										*	@return		UInt32				Location of first occurrance of inChar in this string. npos if not found.
										*/
			UInt32			 			find_first_of(
												const char *inString,
												UInt32 inIndex = 0, 
												const CP_TextEncoder *inTextEncoder = NULL ) const					{ return( find_first_of( CP_String(inString, inTextEncoder), inIndex ) ); }

										/*!
										*	Return the index of the first occurrance of inSting in this string.
										*	
										*	@param		inString			c style (NULL terminated) we are looking for.
										*	@param		inIndex				Starting index to being search at, default is the beginning.
										*	@param		inLength			Number of characters in inString to use in the search.
										*	@param		inTextEncoder		Text encoder used to convert c style string into UTF-16 representation if needed.
										*	
										*	@exception	none
										*	
										*	@return		UInt32				Location of first occurrance of inChar in this string. npos if not found.
										*/
			UInt32			 			find_first_of(
												const char *inString,
												UInt32 inIndex ,
												UInt32 inLength, 
												const CP_TextEncoder *inTextEncoder = NULL) const						{ return( find_first_of( CP_String(inString, inLength, inTextEncoder), inIndex ) ); }

										/*!
										*	Return the index of the first occurrance of inSting in this string.
										*	
										*	@param		inString			Unicode (NULL terminated) we are looking for.
										*	@param		inIndex				Starting index to being search at, default is the beginning.
										*	
										*	@exception	none
										*	
										*	@return		UInt32				Location of first occurrance of inChar in this string. npos if not found.
										*/
			UInt32			 			find_first_of(
												const CPUniChar *inString,
												UInt32 inIndex = 0 ) const					{ return( find_first_of( CP_String(inString), inIndex ) ); }

										/*!
										*	Return the index of the first occurrance of inSting in this string.
										*	
										*	@param		inString			Unicode (NULL terminated) we are looking for.
										*	@param		inIndex				Starting index to being search at, default is the beginning.
										*	@param		inLength			Number of characters in inString to use in the search.
										*	
										*	@exception	none
										*	
										*	@return		UInt32				Location of first occurrance of inString in this string. npos if not found.
										*/
			UInt32			 			find_first_of(
												const CPUniChar *inString,
												UInt32 inIndex,
												UInt32 inLength) const						{ return( find_first_of( CP_String(inString, inLength), inIndex ) ); }

										/*!
										*	Return the index of the first occurrance of inSting in this string.
										*	
										*	@param		inString			CP_String we are looking for.
										*	@param		inIndex				Starting index to being search at, default is the beginning.
										*	
										*	@exception	none
										*	
										*	@return		UInt32				Location of first occurrance of inString in this string. npos if not found.
										*/
			UInt32			 			find_first_of(
												const CP_String& inString,
												UInt32 inIndex = 0 ) const;

										/*!
										*	Return the index of the last occurrance of inChar in this string. The search is backwards in the string
										*	
										*	@param		inString			Unicode character we are looking for.
										*	@param		inIndex				Starting index to being search at, default is the end of the string.
										*	
										*	@exception	none
										*	
										*	@return		UInt32				Location of last occurrance of inChar in this string. npos if not found.
										*/
			UInt32			 			find_last_of(
												CPUniChar inChar,
												UInt32 inIndex = npos ) const				{ return( find_last_of( CP_String(1, inChar), inIndex ) ); }

										/*!
										*	Return the index of the last occurrance of inChar in this string.  The search is backwards in the string
										*	
										*	@param		inString			c style (NULL terminated) string we are looking for.
										*	@param		inIndex				Starting index to being search at, default is the end of the string.
										*	@param		inTextEncoder		Text encoder used to convert c style string into UTF-16 representation if needed.
										*	
										*	@exception	none
										*	
										*	@return		UInt32				Location of last occurrance of inString in this string. npos if not found.
										*/
			UInt32			 			find_last_of(
												const char *inString,
												UInt32 inIndex = npos, 
												const CP_TextEncoder *inTextEncoder = NULL ) const				{ return( find_last_of( CP_String(inString, inTextEncoder), inIndex ) ); }
	

										/*!
										*	Return the index of the last occurrance of inString in this string. The search is backwards in the string
										*	
										*	@param		inString			c style (NULL terminated) string we are looking for.
										*	@param		inIndex				Starting index to being search at, default is the end of the string.
										*	@param 		inLength			Number of characters from inString to use in search.
										*	@param		inTextEncoder		Text encoder used to convert c style string into UTF-16 representation if needed.
										*	
										*	@exception	none
										*	
										*	@return		UInt32				Location of last occurrance of inString in this string. npos if not found.
										*/
			UInt32			 			find_last_of(
												const char *inString,
												UInt32 inIndex,
												UInt32 inLength, 
												const CP_TextEncoder *inTextEncoder = NULL ) const						{ return( find_last_of( CP_String(inString, inLength, inTextEncoder), inIndex ) ); }


										/*!
										*	Return the index of the last occurrance of inString in this string. The search is backwards in the string
										*	
										*	@param		inString			Unicode (NULL terminated) string we are looking for.
										*	@param		inIndex				Starting index to being search at, default is the end of the string.
										*	
										*	@exception	none
										*
										*	@return		UInt32				Location of last occurrance of inString in this string. npos if not found.
										*/
			UInt32			 			find_last_of(
												const CPUniChar *inString,
												UInt32 inIndex = npos ) const				{ return( find_last_of( CP_String(inString), inIndex ) ); }


										/*!
										*	Return the index of the last occurrance of inString in this string. The search is backwards in the string
										*	
										*	@param		inString			Unicode (NULL terminated) string we are looking for.
										*	@param		inIndex				Starting index to being search at, default is the end of the string.
										*	@param		inLength			Number of characters from inString to use in search.
										*	
										*	@exception	none
										*	
										*	@return		UInt32				Location of last occurrance of inString in this string. npos if not found.
										*/
			UInt32			 			find_last_of(
												const CPUniChar *inString,
												UInt32 inIndex,
												UInt32 inLength ) const						{ return( find_last_of( CP_String(inString, inLength), inIndex ) ); }


										/*!
										*	Return the index of the last occurrance of inString in this string.	The search is backwards in the string
										*	
										*	@param		inString			CP_String string we are looking for.
										*	@param		inIndex				Starting index to being search at, default is the end of the string.
										*	
										*	@exception	none
										*	
										*	@return		UInt32				Location of last occurrance of inString in this string. npos if not found.
										*/
			UInt32			 			find_last_of(
												const CP_String& inString,
												UInt32 inIndex = npos ) const;

										/*!
										*	Return the index of the first occurrance of inChar that is not in this string.
										*	
										*	@param		inChar				Unicode character string we are looking for.
										*	@param		inIndex				Starting index to being search at, default is the beginning of the string.
										*	
										*	@exception	none
										*	
										*	@return		UInt32				Location of first occurrance that is not in this string. npos if not found.
										*/
			UInt32			 			find_first_not_of(
												CPUniChar inChar,
												UInt32 inIndex = 0 ) const					{ return( find_first_not_of( CP_String(1, inChar), inIndex ) ); }

										/*!
										*	Return the index of the first occurrance of inString that is not in this string.
										*	
										*	@param		inString			c style (NULL terminated) character string we are looking for.
										*	@param		inIndex				Starting index to being search at, default is the beginning of the string.
										*	@param		inTextEncoder		Text encoder used to convert c style string into UTF-16 representation if needed.
										*	
										*	@exception	none
										*	
										*	@return		UInt32				Location of first occurrance that is not in this string. npos if not found.
										*/
			UInt32			 			find_first_not_of(
												const char *inString,
												UInt32 inIndex = 0, 
												const CP_TextEncoder *inTextEncoder = NULL) const					{ return( find_first_not_of( CP_String(inString, inTextEncoder), inIndex ) ); }

										/*!
										*	Return the index of the first occurrance of inString that is not in this string.
										*	
										*	@param		inString			c style (NULL terminated) character string we are looking for.
										*	@param		inIndex				Starting index to being search at, default is the beginning of the string.
										*	@param		inLength			Number of characters from passed in string to use in search.
										*	@param		inTextEncoder		Text encoder used to convert c style string into UTF-16 representation if needed.
										*	
										*	@exception	none
										*	
										*	@return		UInt32				Location of first occurrance that is not in this string. npos if not found.
										*/
			UInt32			 			find_first_not_of(
												const char *inString,
												UInt32 inIndex ,
												UInt32 inLength,
												const CP_TextEncoder *inTextEncoder = NULL) const						{ return( find_first_not_of( CP_String(inString, inLength, inTextEncoder), inIndex ) ); }

										/*!
										*	Return the index of the first occurrance of inString that is not in this string.
										*	
										*	@param		inString			Unicode (NULL terminated) character string we are looking for.
										*	@param		inIndex				Starting index to being search at, default is the beginning of the string.
										*	
										*	@exception	none
										*	
										*	@return		UInt32				Location of first occurrance that is not in this string. npos if not found.
										*/
			UInt32			 			find_first_not_of(
												const CPUniChar *inString,
												UInt32 inIndex = 0 ) const					{ return( find_first_not_of( CP_String(inString), inIndex ) ); }

										/*!
										*	Return the index of the first occurrance of inString that is not in this string.
										*	
										*	@param		inString			Unicode (NULL terminated) character string we are looking for.
										*	@param		inIndex				Starting index to being search at, default is the beginning of the string.
										*	@param		inLength			Number of characters from passed in string to use in search.
										*	
										*	@exception	none
										*	
										*	@return		UInt32				Location of first occurrance that is not in this string. npos if not found.
										*/
			UInt32			 			find_first_not_of(
												const CPUniChar *inString,
												UInt32 inIndex,
												UInt32 inLength) const						{ return( find_first_not_of( CP_String(inString, inLength), inIndex ) ); }

										/*!
										*	Return the index of the first occurrance of inString that is not in this string.
										*	
										*	@param		inString			CP_String character string we are looking for.
										*	@param		inIndex				Starting index to being search at, default is the beginning of the string.
										*	
										*	@exception	none
										*	
										*	@return		UInt32				Location of first occurrance that is not in this string. npos if not found.
										*/
			UInt32			 			find_first_not_of(
												const CP_String& inString,
												UInt32 inIndex = 0 ) const;

										/*!
										*	Return the index of the last occurrance of inChar that is not in this string. The search is backwards in the string
										*	
										*	@param		inChar				Unicode character we are looking for.
										*	@param		inIndex				Starting index to being search at, default is the end of the string.
										*	
										*	@exception	none
										*	
										*	@return		UInt32				Location of last occurrance that is not in this string.
										*/
			UInt32			 			find_last_not_of(
												CPUniChar inChar,
												UInt32 inIndex = npos ) const				{ return( find_last_not_of( CP_String(1, inChar), inIndex ) ); }

										/*!
										*	Return the index of the last occurrance of inString that is not in this string.	 The search is backwards in the string
										*	
										*	@param		inString			c style (NULL terminated) string we are looking for.
										*	@param		inIndex				Starting index to being search at, default is the end of the string.
										*	@param		inTextEncoder		Text encoder used to convert c style string into UTF-16 representation if needed.
										*	
										*	@exception	none
										*	
										*	@return		UInt32				Location of last occurrance that is not in this string.
										*/
			UInt32			 			find_last_not_of(
												const char *inString,
												UInt32 inIndex = npos,
												const CP_TextEncoder *inTextEncoder = NULL) const				{ return( find_last_not_of( CP_String(inString, inTextEncoder), inIndex ) ); }

										/*!
										*	Return the index of the last occurrance of inString that is not in this string. The search is backwards in the string
										*	
										*	@param		inString			c style (NULL terminated) string we are looking for.
										*	@param		inIndex				Starting index to being search at, default is the end of the string.
										*	@param		inLength			Number of characters from inString to use in search.
										*	@param		inTextEncoder		Text encoder used to convert c style string into UTF-16 representation if needed.
										*	
										*	@exception	none
										*	
										*	@return		UInt32				Location of last occurrance that is not in this string.
										*/
			UInt32			 			find_last_not_of(
												const char *inString,
												UInt32 inIndex,
												UInt32 inLength,
												const CP_TextEncoder *inTextEncoder = NULL) const						{ return( find_last_not_of( CP_String(inString, inLength, inTextEncoder), inIndex ) ); }

										/*!
										*	Return the index of the last occurrance of inString that is not in this string. The search is backwards in the string
										*	
										*	@param		inString			Unicode (NULL terminated) string we are looking for.
										*	@param		inIndex				Starting index to being search at, default is the end of the string.
										*	
										*	@exception	none
										*	
										*	@return		UInt32				Location of last occurrance that is not in this string.
										*/
			UInt32			 			find_last_not_of(
												const CPUniChar *inString,
												UInt32 inIndex = npos ) const				{ return( find_last_not_of( CP_String(inString), inIndex ) ); }

										/*!
										*	Return the index of the last occurrance of inString that is not in this string. The search is backwards in the string
										*	
										*	@param		inString			Unicode (NULL terminated) string we are looking for.
										*	@param		inIndex				Starting index to being search at, default is the end of the string.
										*	@param 		inLength			Number of characters in string to use in search.
										*	
										*	@exception	none
										*	
										*	@return		UInt32				Location of last occurrance that is not in this string.
										*/
			UInt32			 			find_last_not_of(
												const CPUniChar *inString,
												UInt32 inIndex,
												UInt32 inLength ) const						{ return( find_last_not_of( CP_String(inString, inLength), inIndex ) ); }

										/*!
										*	Return the index of the last occurrance of inString that is not in this string.	The search is backwards in the string
										*	
										*	@param		inString			CP_String string we are looking for.
										*	@param		inIndex				Starting index to being search at, default is the end of the string.
										*	
										*	@exception	none
										*	
										*	@return		UInt32				Location of last occurrance that is not in this string.
										*/
			UInt32			 			find_last_not_of(
												const CP_String& inString,
												UInt32 inIndex = npos ) const;
	//@}

	//! @name Inserting
	//@{
			CP_String& 					insert(
												UInt32 inIndex,
												CPUniChar inChar );
			CP_String& 					insert(
												UInt32 inIndex,
												UInt32 inCount,
												CPUniChar inChar );
			CP_String& 					insert(
												UInt32 inIndex,
												const char *inString,
											    const CP_TextEncoder *inTextEncoder = NULL);
			CP_String& 					insert(
												UInt32 inIndex,
												const char *inString,
												UInt32 inLength,
											    const CP_TextEncoder *inTextEncoder = NULL);
			CP_String& 					insert(
												UInt32 inIndex,
												const CPUniChar *inString );
			CP_String& 					insert(
												UInt32 inIndex,
												const CPUniChar *inString,
												UInt32 inLength );
			CP_String& 					insert(
												UInt32 inIndex,
												const CP_String& inString,
												UInt32 inStartIndex = 0,
												UInt32 inLength = npos );
	//@}

	//! @name Replacing
	//@{
			CP_String& 					replace(
												UInt32 inIndex,
												UInt32 inCount,
												UInt32 inCharCount,
												CPUniChar inChar );
			CP_String& 					replace(
												UInt32 inIndex,
												UInt32 inCount,
												const char *inString,
												const CP_TextEncoder *inTextEncoder = NULL);
			CP_String& 					replace(
												UInt32 inIndex,
												UInt32 inCount,
												const char *inString,
												UInt32 InLength,
												const CP_TextEncoder *inTextEncoder = NULL);
			CP_String& 					replace(
												UInt32 inIndex,
												UInt32 inCount,
												const CPUniChar *inString );
			CP_String& 					replace(
												UInt32 inIndex,
												UInt32 inCount,
												const CPUniChar *inString,
												UInt32 inLength );
			CP_String& 					replace(
												UInt32 inIndex,
												UInt32 inCount,
												const CP_String& inString,
												UInt32 inStartIndex = 0,
												UInt32 inLength = npos );
	//@}

	//! @name Size
	//@{
			UInt32 						length() const;
			UInt32						size() const { return length(); }

			void 						resize( UInt32 inNewSize );
			void 						resize(
												UInt32 inNewSize,
												CPUniChar inChar );

			UInt32						capacity() const;
			
										/*!
										*	Return true if this string contains 0 characters.
										*	
										*	@exception	none
										*	
										*	@return		bool const			True if length == 0
										*/
			bool						Empty() const										{ return( length() == 0 ); }
										/*!
										 *	Return true if this string contains 0 characters.
										 *	
										 *	@exception	none
										 *	
										 *	@return		bool const			True if length == 0
										 */
			bool						empty() const										{ return( length() == 0 ); }
			void 						clear();
	//@}

	//! @name Iterators
	//@{
			iterator 					begin();
			const_iterator 				begin() const;

			iterator 					end();
			const_iterator 				end() const;

			#if ! TARGET_CC_MSVC_6 && ! TARGET_CC_MSVC_2002
				reverse_iterator 		rbegin();
				const_reverse_iterator 	rbegin() const;

				reverse_iterator 		rend();
				const_reverse_iterator 	rend() const;
			#endif
	//@}

	//! @name Misc
	//@{
										/*!
										*	Return a string containing a portion of this string.
										*	
										*	@param		inIndex				Starting index to return. Defaults to beginning of string.
										*	@param		inLength			Number of characters to return. Default is to use entire string.
										*	
										*	@exception	none
										*
										*	@return		CP_String const		String containing a the substring of this string.
										*/
			CP_String 					substr(
												UInt32 inIndex = 0,
												UInt32 inCount = npos ) const				{ return( CP_String( *this, inIndex, inCount ) ); }

			UInt32 						copy(
												char *inOutBuffer,
												UInt32 inBufferSize,
											    UInt32 inCount,
												UInt32 *outUsedBufferSize,
												UInt32 inIndex = 0,
												const CP_TextEncoder *inTextEncoder = NULL,
												bool inLossy=false) const;
			UInt32 						copy(
												CPUniChar *inOutBuffer,
												UInt32 inCount,
												UInt32 inIndex = 0 ) const;

    		void 						swap( CP_String& inString );

			void						reserve( UInt32 inCapacity );
			#if ! TARGET_API_MAC_CARBON

				static void 			SetDefaultTextEncoder( CP_TextEncoder *inTextEncoder );

			private:
				void					CreateDefaultTextEncoder() const;

				void					ResizeCharacterBuffer(
												UInt32 inNewLength,
												bool inCopyData = false );	
				void					AssertUniqueStringRef();
				
			#endif	// ! TARGET_API_MAC_CARBON
	//@}

	// class data
protected:
	mutable char						*fCStringDataPtr;							/*!< Pointer to buffer returned for calls to c_str(). */
										
	mutable UInt32						fCStringDataCapacity;						/*!< Amount of data our c style string member can hold (excluding terminating zero character). */
	
	mutable bool						fDirty;										/*!< Flag to indicate if our return buffers need to be rebuilt. */
	
	#if TARGET_OS_MAC && TARGET_API_MAC_CARBON
		CFStringRef						fImmutableString;							/*!< CoreFoundation CFString holding our string data. Only one of \ref fImmutableString and \ref fMutableString are != \a NULL at any time, depending on the mutability of the string.*/
		CFMutableStringRef				fMutableString;								/*!< CoreFoundation CFString holding our string data. Only one of \ref fImmutableString and \ref fMutableString are != \a NULL at any time, depending on the mutability of the string.*/
										
		mutable CPUniChar				*fStringData;								/*!< Pointer to buffer returned for calls to w_str(). */

		mutable UInt32					fStringDataCapacity;						/*!< Number of characters our fStringData member can hold (excluding terminating zero character). */
	#else
		StringRef						fStringRef;									/*!< Our string reference that is storing the real data */
		static CP_TextEncoder			*gDefaultTextEncoder;						/*!< Default CP_TextEncoder to use. */
										
	#endif	// TARGET_API_MAC_CARBON && TARGET_OS_MAC

//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_String( CFStringRef inStringRef );
										CP_String( CFMutableStringRef inStringRef );
			explicit					CP_String( const CP_ResourcePtr<CFStringRef>& inStringRef );
			explicit					CP_String( const CP_ResourcePtr<CFMutableStringRef>& inStringRef );
	//@}

	//! @name Conversion
	//@{
			CP_String&					operator = ( CFStringRef inStringRef );
			CP_String&					operator = ( const CP_ResourcePtr<CFStringRef>& inStringRef );
			CP_String&					operator = ( CFMutableStringRef inStringRef );
			CP_String&					operator = ( const CP_ResourcePtr<CFMutableStringRef>& inStringRef );

										/*!
										*	Return the CFStringRef containing the string data.
										*	
										*	@exception	none
										*
										*	@return		CFStringRef			The CFStringRef for our strings data.
										*/
			CFStringRef					GetCFStringRef() const;

										/*!
										*	Return the CFMutableStringRef containing the string data.
										*	
										*	@exception	CP_OSErrorException(coreFoundationUnknownErr) raised if converting immutable string to mutable string fails.
										*
										*	@return		CFMutableStringRef	The CFMutableStringRef for our strings data.
										*/
			CFMutableStringRef			GetMutableCFStringRef()								{ AssertUniqueMutableCFStringRef(); return fMutableString; }

										/*!
										*	Return the CFStringRef containing the string data as a CP_ResourcePtr wrapped
										*	resource.
										*	
										*	@exception	none
										*
										*	@return		CP_ResourcePtr const	A CP_ResourcePtr that contains the CFStringRef for the string.
										*/
			CP_ResourcePtr<CFStringRef>			GetCFStringRefPtr() const					{ return( CP_ResourcePtr<CFStringRef>(GetCFStringRef(), false) ); }

										/*!
										*	Return the CFMutableStringRef containing the string data as a CP_ResourcePtr wrapped
										*	resource.
										*	
										*	@exception	CP_OSErrorException(coreFoundationUnknownErr) raised if converting immutable string to mutable string fails.
										*
										*	@return		CP_ResourcePtr const	A CP_ResourcePtr that contains the CFMutableStringRef for the string.
										*/
			CP_ResourcePtr<CFMutableStringRef>	GetMutableCFStringRefPtr()					{ AssertUniqueMutableCFStringRef(); return( CP_ResourcePtr<CFMutableStringRef>(fMutableString, false) ); }
	//@}

	//! @name Setters
	//@{
			void						Set(
												const unsigned char *inPString,
												CFStringEncoding inEncoding );
	//@}

	//! @name Substring
	//@{
			void						Substring( CFRange inRange, CPUniChar *inOutBuffer ) const;
	//@}

protected:
			void						AssertUniqueMutableCFStringRef();
#endif

#if TARGET_OS_WIN32
	#if !TARGET_API_MAC_CARBON
	public:
		//! @name Constructor / Destructor
		//@{
		//! Constructor.
			explicit					CP_String( StringRef inStringRef );
		//@}
	#endif // !TARGET_API_MAC_CARBON
#endif

#if TARGET_OS_LINUX
	public:
		//! @name Constructor / Destructor
		//@{
		//! Constructor.
			explicit					CP_String( StringRef inStringRef );
		//@}
#endif
};

/*!
*	@class		ParameterValues
*	@brief		A helper struct used to hold the parameters from the stack to CP_String::StringWithFormatAndArguments
*/

union ParameterValues
{
	char                 theChar;
	CP_String::StringRef theStringRef;
	double               theDouble;
	void                *thePointer;
	SInt64               theInt;
	UInt64               theUInt;
};

/*!
*	@class		CP_FormatAlex
*	@brief		A class for parsing and analyzing the format string 
*	
*				The CP_FormatAlex class parses the format string (same format string used by
*				printf or sprintf) and breaks it into tokens.
*/

class CP_FormatAlex
{
	public:
		enum Tokens
		{
			Token_Unknown       = -1,
			Token_EOS           =  0,
			Token_Comment       =  1,     //<! none of the known types, just the characters as they are in the format string
			Token_CP_String,              //<! CPLAT::CP_String like '@'
			Token_Character,              //<! character (like 'c' or 'C')
			Token_DecimalInteger,         //<! decimal integer (like 'd' or 'i')
			Token_OctalInteger,           //<! unsigned octal integer (like 'o')
			Token_UnsignedDecimalInteger, //<! unsigned decimal integer (like 'u')
			Token_UnsignedHexaInteger,    //<! unsigned hexadecumal integer (like 'x')
			Token_UnsignedCAPSHexaInteger,//<! unsigned hexadecimal integer with capitals (like 'X')
			Token_Double,                 //<! double (like 'e')
			Token_CAPSDouble,             //<! double with capital E (like 'E')
			Token_Float,                  //<! double like 'f'
			Token_FloatG,                 //<! double like 'g'
			Token_FloatCAPSG,             //<! double like 'G'
			Token_Pointer,                //<! pointer like 'p'
			Token_String                  //<! string like 's' or 'S'
		};
		
		enum Modifiers
		{
			Modifier_None = 0,
			Modifier_Char,      //<! like 'hh' it's in fact "short short"
			Modifier_Short,     //<! like 'h',
			Modifier_Long,      //<! like 'l', or 'I32'
			Modifier_LongLong,  //<! like 'll' or 'I64'
			Modifier_LongDouble,//<! like 'L'
			Modifier_t,         //<! like 't'
			Modifier_j,         //<! like 'j'
			Modifier_Size       //<! like 'z'
		};
		
		enum Flags
		{
			Flag_None        =      0,
			Flag_LeftAlign   = 0x0001,    //<! like '-'
			Flag_SignPrefix  = 0x0002,    //<! like '+'
			Flag_ZeroPrefix  = 0x0004,    //<! like '0'
			Flag_BlankPrefix = 0x0008,    //<! like ' '
			Flag_CrossPrefix = 0x0010,    //<! like '#'
			Flag_Thousands   = 0x0020     //<! like '''
		};
		
		/*!
		*	@class		CP_FormatToken
		*	@brief		A class that holds all the parameters of the token found in the format string
		*/
		class CP_FormatToken
		{
			friend class CP_FormatAlex;
			friend class CP_String;
			public:
				CP_FormatToken()
				{
					fTokenFlags        = 0;
					fTokenModifiers    = Modifier_None;
					fTokenType         = Token_Unknown;
					fParameterPosition = 0;    // 0 means "not used"
					fTokenWidth        = -1;   // -1 means "not used", -2 means it's * which will be from parameters list
					fTokenPrecision    = -1;   // -1 means "not used", -2 means it's * which will be from parameters list
				}
				
				CP_FormatToken( const CP_FormatToken& rhs )
				{
					fTokenFlags        = rhs.fTokenFlags;
					fTokenModifiers    = rhs.fTokenModifiers;
					fTokenFace         = rhs.fTokenFace;
					fTokenType         = rhs.fTokenType;
					fParameterPosition = rhs.fParameterPosition;
					fTokenWidth        = rhs.fTokenWidth;
					fTokenPrecision    = rhs.fTokenPrecision;
					
				}
				
				CP_FormatToken& operator = ( const CP_FormatToken& rhs )
				{
					fTokenFlags        = rhs.fTokenFlags;
					fTokenModifiers    = rhs.fTokenModifiers;
					fTokenFace         = rhs.fTokenFace;
					fTokenType         = rhs.fTokenType;
					fParameterPosition = rhs.fParameterPosition;
					fTokenWidth        = rhs.fTokenWidth;
					fTokenPrecision    = rhs.fTokenPrecision;
					
					return *this;
				}
				
				void Reset()
				{
					fTokenFlags        = 0;
					fTokenModifiers    = Modifier_None;
					fTokenType         = Token_Unknown;
					fParameterPosition = 0;
					fTokenWidth        = -1;   // -1 means "not used"
					fTokenPrecision    = -1;   // -1 means "not used"
					fTokenFace.clear();
				}

		//! @name Accessors
		//@{
				UInt32     GetTokenFlags() const { return fTokenFlags; }
				Modifiers  GetTokenModifier() const { return fTokenModifiers; }
				const CP_String& GetTokenFace() const { return fTokenFace; }
				Tokens     GetTokenType() const { return fTokenType; }
				int        GetTokenWidth() const { return fTokenWidth; }
				int        GetTokenPrecision() const { return fTokenPrecision; }
				UInt32     GetParameterPosition() const { return fParameterPosition; }   //<! get the the parameter order in the argument list (one based)
				const CP_String GetTokenFormatString() const;
		//@}
		
			protected:
				UInt32    fTokenFlags;
				Modifiers fTokenModifiers;
				Tokens    fTokenType;
				UInt32    fParameterPosition;  //<! parameter position in argument list (if not used it will specify 0)
				int       fTokenWidth;         //<! minimum number of characters to print
				int       fTokenPrecision;
				CP_String fTokenFace;
		};
	
	public:
	//! @name Constructor/destructor
	//@{
		CP_FormatAlex( const CP_String& formatString );
	//@}	
	
	//! @name Token access
	//@{
		Tokens GetToken();
		const CP_FormatToken& GetTokenObject() const { return fTokenObject; }
	//@}		

	protected:
		CP_String fFormatString;          //<! the format string to parse
		CP_FormatToken fTokenObject;      //<! will keep all the properties of the last parsed token, filled by GetToken() while parsing the input string

	private:
		CPUniChar GetNextInputChar();     //<! extract from the input string the next available character
	
		int       fCurrentParameterOrder; //<! will increment with 1 and will specify the argument position (order) if not using the argument reordering
		CPUniChar fCurrentChar;
		UInt32    fPosition;              //<! current position in the string
};


#if TARGET_API_MAC_CARBON
class CP_String::__iterator_base
{
	friend class CP_String;
protected:
	mutable CFStringInlineBuffer	fInlineBuffer;
	mutable bool					fInlineBufferInitialized;
	const CP_String					*fString;
	CFIndex							fIndex;
	CFIndex							fEndIndex;
	
	__iterator_base(const CP_String& inString, UInt32 inIndex);
public:
	class reference
	{
		CP_String::iterator&	fIterator;
		CP_String::reference	fRealRef;
		friend class CP_String::iterator;
		
		inline reference(CP_String::iterator& inIterator);
	public:
		inline reference& operator = (CPUniChar rhs);
		inline reference& operator = (const reference& rhs);
		
		operator CPUniChar () const { return fRealRef.operator CPUniChar(); }
	};
	
	__iterator_base() : fInlineBufferInitialized(false), fString(NULL) {}
	__iterator_base(const __iterator_base& src)				{ operator = (src); }
	
	__iterator_base& operator ++(void);
	__iterator_base& operator --(void);
	
	__iterator_base& operator += (SInt32 distance);
	__iterator_base& operator -= (SInt32 distance);
	
	SInt32 operator - (const __iterator_base& rhs) const;
	
	__iterator_base& operator = (const __iterator_base& rhs);
	__iterator_base& operator = (const CP_String::iterator& rhs);
	
	bool operator == (const __iterator_base& rhs) const { return fIndex == rhs.fIndex; }
	bool operator != (const __iterator_base& rhs) const { return fIndex != rhs.fIndex; }
	
	CPUniChar operator *() const;
};
class CP_String::const_iterator : public CP_String::__iterator_base, public std::iterator<std::random_access_iterator_tag, CPUniChar, SInt32, CPUniChar*, CPUniChar>
{
	friend class CP_String;
protected:
	const_iterator(const CP_String& inString, UInt32 inIndex) : CP_String::__iterator_base(inString, inIndex) {}
public:
	typedef CPUniChar reference;
	
	const_iterator() {}
	const_iterator(const const_iterator& src) : CP_String::__iterator_base(src) {}
	inline const_iterator(const iterator& src);
	
	const_iterator& operator ++(void)								{ return static_cast<const_iterator&>( CP_String::__iterator_base::operator ++ () ); }
	const_iterator operator ++(int /*i*/)							{ const_iterator tmp(*this); operator ++(); return tmp; }
	const_iterator& operator --(void)								{ return static_cast<const_iterator&>( CP_String::__iterator_base::operator -- () ); }
	const_iterator operator --(int /*i*/)							{ const_iterator tmp(*this); operator ++(); return tmp; }
	
	const_iterator& operator += (SInt32 distance)					{ return static_cast<const_iterator&>( CP_String::__iterator_base::operator += (distance) ); }
	const_iterator& operator -= (SInt32 distance)					{ return static_cast<const_iterator&>( CP_String::__iterator_base::operator -= (distance) ); }
	
	const_iterator operator + (SInt32 distance) const				{ const_iterator newIterator(*this); newIterator += distance; return newIterator; }
	const_iterator operator - (SInt32 distance) const				{ const_iterator newIterator(*this); newIterator -= distance; return newIterator; }
	
	CPUniChar operator *() const									{ return CP_String::__iterator_base::operator * (); }
};
class CP_String::iterator : public CP_String::__iterator_base, public std::iterator<std::random_access_iterator_tag, CPUniChar, SInt32, CPUniChar*, CP_String::__iterator_base::reference>
{
	friend class CP_String;
	friend class CP_String::__iterator_base::reference;

protected:
	iterator(CP_String& inString, UInt32 inIndex) : CP_String::__iterator_base(inString, inIndex) {}
	void UpdateFocusedChar(CPUniChar inUpdatedChar);
	CP_String::reference make_char_reference() { return CP_String::reference(*const_cast<CP_String*>(fString), (UInt32) fIndex); }

public:
	typedef CP_String::__iterator_base::reference	reference;
	
	iterator() {}
	iterator(const iterator& src) : CP_String::__iterator_base(src) {}
	
	iterator& operator ++(void)								{ return static_cast<iterator&>( CP_String::__iterator_base::operator ++ () ); }
	iterator operator ++(int /*i*/)							{ iterator tmp(*this); operator ++(); return tmp; }
	iterator& operator --(void)								{ return static_cast<iterator&>( CP_String::__iterator_base::operator -- () ); }
	iterator operator --(int /*i*/)							{ iterator tmp(*this); operator ++(); return tmp; }
	
	iterator& operator += (SInt32 distance)					{ return static_cast<iterator&>( CP_String::__iterator_base::operator += (distance) ); }
	iterator& operator -= (SInt32 distance)					{ return static_cast<iterator&>( CP_String::__iterator_base::operator -= (distance) ); }
	
	iterator operator + (SInt32 distance) const				{ iterator newIterator(*this); newIterator += distance; return newIterator; }
	iterator operator - (SInt32 distance) const				{ iterator newIterator(*this); newIterator -= distance; return newIterator; }
	
	CPUniChar operator *() const							{ return CP_String::__iterator_base::operator * (); }
	reference operator *()									{ return reference(*this); }
};

CP_String::__iterator_base::reference::reference(CP_String::iterator& inIterator) : fIterator(inIterator), fRealRef(inIterator.make_char_reference()) {}
CP_String::__iterator_base::reference& CP_String::__iterator_base::reference::operator = (CPUniChar rhs) { fRealRef.operator = (rhs); fIterator.UpdateFocusedChar(rhs); return *this; }
CP_String::__iterator_base::reference& CP_String::__iterator_base::reference::operator = (const reference& rhs) { CPUniChar ch(rhs.fRealRef); fRealRef.operator = (ch); fIterator.UpdateFocusedChar(ch); return *this; }
#if ! TARGET_CC_METROWERKS
		//\todo -- this produces a compile error on CodeWarrior
	CP_String::const_iterator::const_iterator(const iterator& src) : CP_String::__iterator_base(src) {}
#endif
inline SInt32 operator - (const CP_String::__iterator_base& lhs, const CP_String::__iterator_base& rhs) { return lhs.operator - (rhs); }

#endif // TARGET_API_MAC_CARBON

#if TARGET_API_MAC_CARBON
	#define CP_STR(x)	CP_String(CFSTR(x))
#else
	#define CP_STR(x)	CP_String(x)
#endif

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_String

