//=========================================================================
//
//	FILE:				CP_TextEncoder.cpp
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Classes for text encoders that are used to translate
//						character data before various text encodings.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 106 $
//						Latest Checkout:  $Date: 2005-03-21 15:03:06 -0800 (Mon, 21 Mar 2005) $
//
//=========================================================================
#ifndef _H_CP_TextEncoder
#define _H_CP_TextEncoder
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_NoCopy.h"
#include "CP_String.h"
#include "CP_Types.h"

CPLAT_Begin_Namespace_CPLAT

class CP_Mutex;

/*! Flags used in the conversion process */
enum
{
	kCP_ErrorOnInvalidCharacters 	= 0x1000,		/*!< Posts an error when an invalid character is encountered */
	kCP_MapStrictChars 				= 0x2000,		/*!< Maps strct characters. */
	kCP_TextIsLeftToRight			= 0x3000,		/*!< Text is formatted left to right. */
	kCP_TextIsRightToLeft			= 0x4000		/*!< Text if formatted right to left. */
};

/*! Encoding type - defines for all platforms to return the type of encoder that match the defines used for CFStringBuiltInEncodings. */
enum
{
  kCP_EncodingInvalidId				= (long)0xFFFFFFFF,
  kCP_EncodingMacRoman				= 0,
  kCP_EncodingWindowsLatin1			= 0x0500, 		/*!< ANSI codepage 1252 */
  kCP_EncodingISOLatin1				= 0x0201, 		/*!< ISO 8859-1 */
  kCP_EncodingNextStepLatin			= 0x0B01, 		/*!< NextStep encoding*/
  kCP_EncodingASCII					= 0x0600, 		/*!< 0..127 (in creating CFString, values greater than 0x7F are treated as corresponding Unicode value) */
  kCP_EncodingUnicode				= 0x0100, 		/*!< kTextEncodingUnicodeDefault  + kTextEncodingDefaultFormat (aka kUnicode16BitFormat) */
  kCP_EncodingUTF8					= 0x08000100,	/*!< kTextEncodingUnicodeDefault + kUnicodeUTF8Format */
  kCP_EncodingNonLossyASCII			= 0x0BFF		/*!< 7bit Unicode variants used by YellowBox & Java */
};

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === CP_TextEncoder ===
#pragma mark -
#endif

/*!
*	@class		CP_TextEncoder
*	@brief		Base class used with the CP_String class to provide conversion between
*				Unicode and ASCII encodings.
*
*	@ingroup	TextHandling
*/
class CP_EXPORT CP_TextEncoder : public CP_NoCopy
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_TextEncoder(
												UInt32 inTextFormat,
												bool inUseCompositeChars = false );
	//! Destructor.
	virtual								~CP_TextEncoder();
	//@}

	//! @name Cloning
	//@{
	virtual	CP_TextEncoder				*Clone() const = 0;
	//@}

	//! @name Converters
	//@{
	virtual	UInt32						ToUTF16(
												const char *inSrcChars,
												UInt32 inSrcLength,
												CPUniChar *inDestChars,
												UInt32 inDestLength ) const;
	virtual	UInt32						FromUTF16(
												const CPUniChar *inSrcChars,
												UInt32 inSrcLength,
												char *inDestChars,
												UInt32 inDestLength ) const;
	//@}

	//! @name Flags
	//@{
			void 						SetConvertToUTF16Flags( UInt32 inFlags = 0 );
			void 						SetConvertFromUTF16Flags( UInt32 inFlags = 0 );
	//@}

	//! @name Getters
	//@{
	virtual	UInt32						GetDestLength(
												const char *inSrcChars,
												UInt32 inSrcLength ) const;
	virtual	UInt32						GetDestLength(
												const CPUniChar *inSrcChars,
												UInt32 inSrcLength ) const;
	virtual	UInt32						GetEncodingType() const								{ return( kCP_EncodingASCII ); }
	//@}

	//! @name Name
	//@{
										/*! Return the name of this encoder as a CPUniChar * string. */
			const CPUniChar				*GetEncoding() const								{ return( fEncoding ); }
	//@}

protected:
	
	// class data
protected:
	UInt32								fTextFormat;				/*! Format of the text. */
	bool								fUseCompositeCharacters;	/*! Flag to indicate if we are using composite characters. */

	UInt32 								fConvertToUTF16Flags;		/*! Flag used in converting to UTF16. */
	UInt32 								fConvertFromUTF16Flags;		/*! Flag used in converting from UTF16. */
	
	const CPUniChar						*fEncoding;					/*! The encoding being used, expressed as a CPUniChar *string. */

	#if TARGET_OS_MAC
		mutable TextToUnicodeInfo		fTextToUnicodeInfo;			/*! TextToUnicode info structure used on the Mac. */
		mutable UnicodeToTextInfo		fUnicodeToTextInfo;			/*! UnicodeToTexUnicode info structure used on the Mac. */
	#endif

//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
protected:
	//! @name Testers
	//@{
			bool						HasTextEncodingConverter() const;
			SInt32						GetTECVersion() const;
	//@}
#endif

#if TARGET_OS_WIN32
#endif
};

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === CP_MacRomanTextEncoder ===
#pragma mark -
#endif

/*!
*	@class		CP_MacRomanTextEncoder
*	@brief		CP_TextEncoder class to provide text encoded in the MacOS Roman encoding.
*
*	@ingroup	TextHandling
*/
class CP_EXPORT CP_MacRomanTextEncoder : public CP_TextEncoder
{
public:
	//! Accessor for shared text encoder object
	static const CP_MacRomanTextEncoder* SharedTextEncoder();
	static CP_Mutex& SharedTextEncoderMutex();
	
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_MacRomanTextEncoder( bool inUseCompositeChars = false );
	//! Destructor.
	virtual								~CP_MacRomanTextEncoder();
	//@}

	//! @name Cloning
	//@{
	virtual	CP_TextEncoder				*Clone() const;
	//@}

	//! @name Getters
	//@{
	virtual	UInt32						GetEncodingType() const									{ return( kCP_EncodingMacRoman ); }
	//@}

#if TARGET_OS_WIN32
	//! @name Converters
	//@{
	virtual	UInt32						ToUTF16(
												const char *inSrcChars,
												UInt32 inSrcLength,
												CPUniChar *inDestChars,
												UInt32 inDestLength ) const;
	virtual	UInt32						FromUTF16(
												const CPUniChar *inSrcChars,
												UInt32 inSrcLength,
												char *inDestChars,
												UInt32 inDestLength ) const;
	//@}

	//! @name Getters
	//@{
	virtual	UInt32						GetDestLength(
												const char *inSrcChars,
												UInt32 inSrcLength ) const;
	virtual	UInt32						GetDestLength(
												const CPUniChar *inSrcChars,
												UInt32 inSrcLength ) const;
	//@}
#endif

	// class data
protected:

	#if TARGET_OS_WIN32
		bool							fHasMacCodePage;	/*! Flag to indicate if we have the Mac code page. */
	#endif

//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif
};

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === CP_TextEncoder ===
#pragma mark -
#endif

/*!
*	@class		CP_WindowsLatin1TextEncoder
*	@brief		CP_TextEncoder class to provide text encoded in the Windows Latin1 encoding.
*
*	@ingroup	TextHandling
*/
class CP_EXPORT CP_WindowsLatin1TextEncoder : public CP_TextEncoder
{
public:
	//! Accessor for shared text encoder object
	static const CP_WindowsLatin1TextEncoder* SharedTextEncoder();
	static CP_Mutex& SharedTextEncoderMutex();

	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_WindowsLatin1TextEncoder( bool inUseCompositeChars = false );
	//! Destructor.
	virtual								~CP_WindowsLatin1TextEncoder();
	//@}

	//! @name Cloning
	//@{
	virtual	CP_TextEncoder				*Clone() const;
	//@}

	//! @name Getters
	//@{
	virtual	UInt32						GetEncodingType() const							{ return( kCP_EncodingWindowsLatin1 ); }
	//@}

	// class data
protected:

//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif
};

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === CP_US_ASCIITextEncoder ===
#pragma mark -
#endif

/*!
*	@class		CP_US_ASCIITextEncoder
*	@brief		CP_TextEncoder class to provide text encoded in the US ASCII encoding.
*
*	@ingroup	TextHandling
*/
class CP_EXPORT CP_US_ASCIITextEncoder : public CP_TextEncoder
{
public:
	//! Accessor for shared text encoder object
	static const CP_US_ASCIITextEncoder* SharedTextEncoder();
	static CP_Mutex& SharedTextEncoderMutex();
	
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_US_ASCIITextEncoder( bool inUseCompositeChars = false );
	//! Destructor.
	virtual								~CP_US_ASCIITextEncoder();
	//@}

	//! @name Cloning
	//@{
	virtual	CP_TextEncoder				*Clone() const;
	//@}

	//! @name Getters
	//@{
	virtual	UInt32						GetEncodingType() const								{ return( kCP_EncodingASCII ); }
	//@}

	// class data
protected:

//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
public:
	//! @name Converters
	//@{
	virtual	UInt32						FromUTF16(
												const CPUniChar *inSrcChars,
												UInt32 inSrcLength,
												char *inDestChars,
												UInt32 inDestLength ) const;
	//@}
#endif
};

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === CP_US_ASCIITextEncoder ===
#pragma mark -
#endif

/*!
*	@class		CP_UTF8TextEncoder
*	@brief		CP_TextEncoder class to provide text encoded in UTF-8 encoding.
*
*	@ingroup	TextHandling
*/
class CP_EXPORT CP_UTF8TextEncoder : public CP_TextEncoder
{
public:
	//! Accessor for shared text encoder object
	static const CP_UTF8TextEncoder* SharedTextEncoder();
	static CP_Mutex& SharedTextEncoderMutex();
	
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_UTF8TextEncoder( bool inUseCompositeChars = false );
	//! Destructor.
	virtual								~CP_UTF8TextEncoder();
	//@}

	//! @name Cloning
	//@{
	virtual	CP_TextEncoder				*Clone() const;
	//@}

	//! @name Converters
	//@{
	virtual	UInt32						ToUTF16(
												const char *inSrcChars,
												UInt32 inSrcLength,
												CPUniChar *inDestChars,
												UInt32 inDestLength ) const;
	virtual	UInt32						FromUTF16(
												const CPUniChar *inSrcChars,
												UInt32 inSrcLength,
												char *inDestChars,
												UInt32 inDestLength ) const;
	//@}
												
	//! @name Getters
	//@{
	virtual	UInt32						GetDestLength(
												const char *inSrcChars,
												UInt32 inSrcLength ) const;
	virtual	UInt32						GetEncodingType() const								{ return( kCP_EncodingUTF8 ); }
	//@}

private:
	//! @name Testers
	//@{
			bool						IsValidUTF8ForSystem(
												const char *inSrcChars,
												UInt32 inSrcLength ) const;

			bool						IsValidUTF8(
												const char *inSrcChars,
												UInt32 inSrcLength ) const;

			bool						IsValidUTF16ForSystem(
												const CPUniChar *inSrcChars,
												UInt32 inSrcLength ) const;

			bool						IsValidUTF16(
												const CPUniChar *inSrcChars,
												UInt32 inSrcLength ) const;
	//@}

	// class data
protected:

	bool								fUseSystemEncoder;	/*! Flag to indicate if the encoding is done by the system. */
	
	#if TARGET_OS_MAC
		mutable TECObjectRef			fTo16TEC;			/*! Mac TextEncoding ref to go to UTF16 encoding. */
		mutable TECObjectRef			fFrom16TEC;			/*! Mac TextEncoding ref to go from UTF16 encoding. */
	#endif

	#if TARGET_OS_WIN32
		bool							fHasMacCodePage;	/*! Flag to indicate if we have the Mac code page. */
	#endif

//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif
};

/*! @brief Lock a shared text encoder.

	This convenience macro will lock a mutex regulating access to the shared text encoder of the specified class 
	until the end of the current code block.\n
	Example:
	\verbatim
	if ( needToConvert )
	{
		CP_LockSharedTextEncoder(lock, CP_UTF8TextEncoder);
		convert(CP_UTF8TextEncoder::SharedTextEncoder());
	}
	\endverbatim
	Here, the shared UTF8 text encoder is locked within the \a if block.

	@param lock					The name of the lock variable. Pass a name that's unique within the current code block.
	@param TextEncoderClass		The encoder class whose shared encoder object should be mutex-locked.
*/

#define CP_LockSharedTextEncoder(lock, TextEncoderClass) \
	CP_MutexLock lock(TextEncoderClass::SharedTextEncoderMutex());

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_TextEncoder
