//=========================================================================
//
//	FILE:				CP_StatusBar.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			CP_View subclass to implement a status bar.
//	
//	HISTORY:			1st Version - December 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 38 $
//						Latest Checkout:  $Date: 2005-01-01 16:42:29 -0800 (Sat, 01 Jan 2005) $
//
//=========================================================================
#ifndef _H_CP_StatusBar
#define _H_CP_StatusBar
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_View.h"

CPLAT_Begin_Namespace_CPLAT

#if ! TARGET_OS_WIN32
	#define SBT_OWNERDRAW            0x1000
	#define SBT_NOBORDERS            0x0100
	#define SBT_POPOUT               0x0200
	#define SBT_RTLREADING           0x0400
#endif

// ===================================================================================
//	CP_S_StatusBarPart - internal structure used on Mac to maintain info about a single
//		part in our status bar.
// ===================================================================================
struct CP_S_StatusBarPart
{
	CP_String				fText;
	CP_Rect					fBounds;
	SInt16					fFlags;
	SInt32					fUserData;
	
							CP_S_StatusBarPart();
							CP_S_StatusBarPart( const CP_Rect& inBounds );
							CP_S_StatusBarPart(
									const CP_String& inText,
									const CP_Rect& inBounds,
									SInt16 inFlags = 0,
									SInt32 inUserData = 0 );
							CP_S_StatusBarPart( const CP_S_StatusBarPart& inPartInfo );

	CP_S_StatusBarPart&		operator =( const CP_S_StatusBarPart& inPartInfo );
};

	// typedef
#if ! TARGET_OS_LINUX
	typedef std::vector<CP_S_StatusBarPart, std::allocator<CP_S_StatusBarPart> >					CP_StatusBarPart_Array;
	typedef std::vector<CP_S_StatusBarPart, std::allocator<CP_S_StatusBarPart> >::iterator			CP_StatusBarPart_Iterator;
	typedef std::vector<CP_S_StatusBarPart, std::allocator<CP_S_StatusBarPart> >::reverse_iterator	CP_StatusBarPart_ReverseIterator;
#else
	typedef std::vector<CP_S_StatusBarPart>															CP_StatusBarPart_Array;
	typedef std::vector<CP_S_StatusBarPart>::iterator												CP_StatusBarPart_Iterator;
	typedef std::vector<CP_S_StatusBarPart>::reverse_iterator										CP_StatusBarPart_ReverseIterator;
#endif

/*!
*	@class		CP_StatusBar
*	@brief		CP_View subclass to implement a status bar.
*/
class CP_EXPORT CP_StatusBar : public CPLAT::CP_View
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_StatusBar();
										CP_StatusBar(
												CPLAT::CP_View *inSuperview,
												const CPLAT::CP_Point& inUpperLeft,
												const CPLAT::CP_Size& inContentSize,
												bool inAutoPosition = true,
												bool inAtWindowBottom = true,
												bool inGrowBox = true );
	//! Destructor.
	virtual								~CP_StatusBar() throw();
	//@}
			
	//! @name Persistance
	//@{
	static CP_M_Persist					*PersistCreator();
	//@}

	//! @name Getters
	//@{
			bool						GetBorders( SInt32 *ptArray );
			SInt16						GetNumberParts();
			SInt16						GetParts(
												SInt32 *ptArray,
												SInt16 inNumElements );
			CP_Rect						GetRect( SInt16 inPartNumber );
			CP_String					GetText( SInt16 inPartNumber );
			SInt32						GetTextLength( SInt16 inPartNumber );
	//@}

	//! @name Setters
	//@{
			void						SetMinHeight( SInt16 inMinHeight );
			bool						SetParts(
												SInt32 *ptArray,
												SInt16 inNumElements );
			bool						SetSimpleDisplay( bool inShowSimple );
			bool						SetText(
												const CP_String& inText,
												SInt16 inPartNumber,
												SInt16 inDisplayFlags = 0 );
	//@}

protected:
	//! @name Persistance
	//@{
	virtual	void						OnInitPersist(
												CP_View *inSuperview,
												CP_DataValueList& inDataValueList );
	//@}

	//! @name Parts
	//@{
			CP_S_StatusBarPart			GetPartInfo( SInt16 inPartNumber );
			void						SetPartInfo(
												const CP_S_StatusBarPart& inPartInfo,
												SInt16 inPartNumber );
	//@}

	// class data
protected:
	bool								fAutoPosition;				/*! Flag to indicate if this pane should auto position itself at
																		the bottom of its enclosing window. */
	bool								fHasGrowBox;				/*! Flag to indicate if we need to cover a grow box area. */
	bool								fAtWindowBottom;			/*! Flag to indicate if this pane is at the bottom of a window. */
	bool								fSimpleStyle;				/*! Flag to indicate if this should be a simple style, where there is
																		only 1 area with text displayed. */

	CP_StatusBarPart_Array				*fParts;					/*! Holds information about the various parts of the status bar. */

	SInt16								fVerticalBorder;			/*! Vertical distance between pane edge and beginning of drawing area. */
	SInt16								fHorizontalBorder;			/*! Horizontal distance between pane edge and beginning of drawing area. */
	SInt16								fDividerBorder;				/*! Thickness of border between parts. */

	#if ! TARGET_OS_WIN32
		SInt16							fStatusBarHeight;			/*! Height of the window. */
	#endif
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
protected:
	//! @name Drawing
	//@{
	virtual	void						OnDraw(
												CP_Canvas& inCanvas,
												CP_Region& inUpdateArea );

	virtual	void						OnOwnerDrawPart(
												CP_Canvas& inCanvas,
												const CP_S_StatusBarPart& inPart,
												const CP_Rect& inBounds );
			void						DrawIndividualPart(
												CP_Canvas& inCanvas,
												const CP_S_StatusBarPart& inPart );
	//@}

private:
	//! @name Initialization
	//@{
			void						CreatePlatformImp(
												CP_View *inSuperview,
												const CP_Point& inUpperLeft,
												const CP_Size& inContentSize,
												bool inAutoPosition,
												bool inAtWindowBottom,
												bool inGrowBox );
	//@}

	//! @name Misc
	//@{
			void						CalcWindowPosition();
	//@}
#endif

#if TARGET_OS_WIN32
private:
	//! @name Initialization
	//@{
			void						CreatePlatformImp(
												CP_View *inSuperview,
												const CP_Point& inUpperLeft,
												const CP_Size& inContentSize,
												bool inAutoPosition,
												bool inAtWindowBottom,
												bool inGrowBox );
	//@}
#endif

#if TARGET_OS_LINUX
private:
	//! @name Initialization
	//@{
			void						CreatePlatformImp(
												CP_View *inSuperview,
												const CP_Point& inUpperLeft,
												const CP_Size& inContentSize,
												bool inAutoPosition,
												bool inAtWindowBottom,
												bool inGrowBox );
	//@}
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_StatusBar

