//=========================================================================
//
//	FILE:				CP_Window_Imp_Win.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Windows implemention of the CP_Window class.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 253 $
//						Latest Checkout:  $Date: 2005-08-22 20:25:19 -0700 (Mon, 22 Aug 2005) $
//
//=========================================================================
#ifndef _H_CP_Window_Imp_Win
#define _H_CP_Window_Imp_Win
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

#if TARGET_OS_WIN32

	// CPLAT_Framework
#include "CP_EventHandler.h"
#include "CP_M_Persist.h"
#include "CP_Point.h"
#include "CP_Size.h"
#include "CP_String.h"
#include "CP_Types.h"
#include "CP_View.h"
#include "CP_View_Imp_Win.h"
#include "CP_WindowTypes.h"

CPLAT_Begin_Namespace_CPLAT

/*!
*	@class		CP_Window_Imp
*	@brief		Implementation of the CP_Window class for Windows.
*
*				The implementation provides the CP_Window class under Windows.
*
*	@ingroup	UI
*/
class CP_EXPORT CP_Window_Imp : public CP_View
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_Window_Imp();
	//! Destructor.
	virtual								~CP_Window_Imp() throw();
	//@}
				
	//! @name Appearance
	//@{
			void						Show();
			void						Hide();
	
			void						Select();
	//@}
			
	//! @name Commands
	//@{
	virtual	void						HandleCloseCommand(
												OSStatus& inOutStatus,
												CP_CommandNo& inCommandNo );
	//@}

	//! @name Drawing
	//@{
	virtual	void						HandleDraw( CP_Event_Base& inEvent );
	//@}

	//! @name Getters
	//@{
	virtual	CP_Rect						GetLocalBounds();
	virtual	CP_Rect						GetPortBounds();
	virtual	CP_Size						GetSize() const;

	virtual	OSWindow					GetImpPlatformWindow() const								{ return( fOSWindow ); }

										/*!
										*	Return the minimum size a window can shrink to.
										*	
										*	@exception	none
										*
										*	@return		CP_Size const		The minimum size a window is allowed to shrink to.
										*/
			CP_Size						GetMinSize() const										{ return( fMinSize ); }

										/*!
										*	Return the maximum size a window can grow to.
										*	
										*	@exception	none
										*
										*	@return		CP_Size const		The maximumm size a window is allowed to grow to.
										*/
			CP_Size						GetMaxSize() const										{ return( fMaxSize ); }
	//@}

	//! @name Events
	//@{
										/*!
										*	Returns the CP_EventHandler that handles events for this window.
										*	
										*	@exception	none
										*	
										*	@return		CP_EventHandler&		Returns a reference to our event handler.
										*/
	virtual	CP_EventHandler&			GetEventHandler()											{ return( fEventHandler ); }

	virtual	LRESULT						HandleWinProcMessage(
												HWND inHwnd,
												UINT inIMsg,
												WPARAM inWParam,
												LPARAM inLParam );
			
			void						InstallMouseMovedEventHandler();
	//@}

	//! @name Title
	//@{
			void						SetTitle( const CP_String& inTitle );
			CP_String					GetTitle() const;
	//@}

	//! @name Setters
	//@{
										/*!
										*	Sets the minimum size a window can shrink to.
										*	
										*	@param		inMinSize			The minimum size a window can shrink to.
										*
										*	@exception	none
										*
										*	@return		void
										*/
			void						SetMinSize( const CP_Size inMinSize )					{ fMinSize = inMinSize; }

										/*!
										*	Sets the maximum size a window can grow to.
										*	
										*	@param		inMaxSize			The maximum size a window can grow to.
										*
										*	@exception	none
										*
										*	@return		void
										*/
			void						SetMaxSize( const CP_Size inMaxSize )					{ fMaxSize = inMaxSize; }

										/*!
										*	Sets the Windows HWND that this window object is using for its platform window.
										*
										*	@param		inWindow			The OS window to use.
										*
										*	@exception	none
										*
										*	@return		void
										*/
			void						SetImpPlatformWindow( OSWindow inWindow )				{ fOSWindow = inWindow; }
	//@}

	//! @name Testers
	//@{
	virtual	bool						IsActive() const;
	//@}

	//! @name WindowTypeBase
	//@{	
										/*!
										*	Returns the CP_WindowTypeBase object that controls the appearance and behavior
										*	of the window.
										*
										*	@exception	none
										*
										*	@return		CP_WindowTypeBase	The CP_WindowTypeBase that controls the windows type
										*									and appearance.
										*/
			CP_WindowTypeBase			GetWindowTypeBase()										{ return( fWindowTypeBase ); }

										/*!
										*	Sets the CP_WindowTypeBase object that controls the appearance and behavior
										*	of the window.
										*
										*	@param		inBase				The CP_WindowTypeBase that controls the type and appearance
										*									of the window.
										*
										*	@exception	none
										*
										*	@return		void
										*/
			void						SetWindowTypeBase( const CP_WindowTypeBase& inBase )	{ fWindowTypeBase = inBase; }
	//@}

protected:
	//! @name Initialization
	//@{
	virtual	void						CreatePlatformImp(
												const CP_Point& inUpperLeft,
												const CP_Size& inContentSize,
												const CP_String& inWindowTitle,
												const CP_WindowTypeBase& inWindowType );
	//@}

	//! @name Events
	//@{
			void						InitEventHandler( CP_EventHandler& inEventHandler );

	virtual void						OnClose();
			void						HandleBoundsChangedEvent( CP_Event_Base& inEvent );
			void						HandleControlColorEvent( CP_Event_Base& inEvent );
	//@}

private:
	//! @name Commands
	//@{
			void						CommandUpdate( CP_Event_Base& inEvent );
			void						CommandProcess( CP_Event_Base& inEvent );
	//@}

	//! @name Events
	//@{
			void						HandleCloseEvent( CP_Event_Base& inEvent );
			void						HandleSizingEvent( CP_Event_Base& inEvent );
			void						HandleMouseMovedEvent( CP_Event_Base& inEvent );
	//@}

	// class data
protected:
	OSWindow							fOSWindow;				/*! Platforms window reference of the window. */
	CP_DataValueSignal					fWindowClosingSignal;	/*! Signal sent when the window is closing. */

	CP_WindowTypeBase					fWindowTypeBase;		/*! Contains information about the window style. */

	CP_Size								fMinSize;				/*! Minimum size a window can shrink to. */
	CP_Size								fMaxSize;				/*! Maximum size a window can grow to. */

	WNDPROC								fnOldWndProc;			/*! Previous Windows window def proc called in the event handler. */

	CP_Rect								fPreviousBounds;

private:
	CP_EventHandler						fEventHandler;			/*! CPLAT event handler object that handles all window events. */
};

CPLAT_End_Namespace_CPLAT

#endif	// TARGET_OS_WIN32

#endif	// _H_CP_Window_Imp_Win
